"use strict";
// -*- mode: typescript; indent-tabs-mode: nil; js-basic-offset: 4 -*-
//
// This file is part of ThingTalk
//
// Copyright 2017-2020 The Board of Trustees of the Leland Stanford Junior University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: Giovanni Campagna <gcampagn@cs.stanford.edu>
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const Ast = __importStar(require("../ast"));
const date_utils_1 = require("../utils/date_utils");
const TERMINAL_IDS = { "0": 16, "1": 17, " 0PAD": 0, " 1EOF": 1, " 2START": 2, "\"": 3, "\"\"": 4, "#[": 5, "$dialogue": 6, "%": 7, "(": 8, ")": 9, "*": 10, "**": 11, "+": 12, ",": 13, "-": 14, "/": 15, ":": 18, ";": 19, "<=": 20, "=": 21, "==": 22, "=>": 23, "=~": 24, ">=": 25, "ATTRIBUTE_NAME": 26, "CLASS_STAR": 27, "CONTEXT_REF": 28, "CURRENCY": 29, "CURRENCY_CODE": 30, "Currency": 31, "DATE": 32, "DEVICE": 33, "DURATION": 34, "Date": 35, "EMAIL_ADDRESS": 36, "ENTITY_TYPE": 37, "ENUM": 38, "FUNCTION": 39, "GENERIC_ENTITY": 40, "HASHTAG": 41, "LITERAL_INTEGER": 42, "LITERAL_TIME": 43, "LOCATION": 44, "Location": 45, "MEASURE": 46, "NUMBER": 47, "PARAM_NAME": 48, "PATH_NAME": 49, "PHONE_NUMBER": 50, "PICTURE": 51, "QUOTED_STRING": 52, "RecurrentTimeSpecification": 53, "SLOT": 54, "SPECIAL": 55, "TIME": 56, "UNIT": 57, "URL": 58, "USERNAME": 59, "WORD": 60, "[": 61, "]": 62, "aggregate": 63, "and": 64, "answer": 65, "asc": 66, "attimer": 67, "avg": 68, "base": 69, "beginDate": 70, "beginTime": 71, "bookkeeping": 72, "choice": 73, "compute": 74, "confirm": 75, "contains": 76, "contains~": 77, "count": 78, "dayOfWeek": 79, "desc": 80, "distance": 81, "edge": 82, "endDate": 83, "endTime": 84, "end_of": 85, "ends_with": 86, "error": 87, "event": 88, "executor": 89, "expiration_date": 90, "false": 91, "filter": 92, "frequency": 93, "in_array": 94, "in_array~": 95, "interval": 96, "join": 97, "let": 98, "location:": 99, "location:current_location": 100, "location:home": 101, "location:work": 102, "max": 103, "min": 104, "monitor": 105, "more": 106, "new": 107, "not": 108, "notify": 109, "now": 110, "of": 111, "on": 112, "or": 113, "policy": 114, "prefix_of": 115, "results": 116, "sort": 117, "special": 118, "start_of": 119, "starts_with": 120, "subtract": 121, "suffix_of": 122, "sum": 123, "time": 124, "time:evening": 125, "time:morning": 126, "timer": 127, "true": 128, "undefined": 129, "{": 130, "}": 131, "~=": 132, "~contains": 133, "~in_array": 134 };
const RULE_NON_TERMINALS = [48, 48, 48, 48, 48, 48, 49, 49, 49, 44, 44, 44, 44, 38, 38, 37, 37, 37, 41, 41, 40, 39, 39, 39, 39, 42, 42, 43, 43, 70, 70, 70, 66, 66, 67, 67, 67, 68, 68, 68, 68, 75, 75, 75, 75, 9, 5, 5, 5, 5, 77, 81, 81, 81, 81, 81, 81, 81, 81, 81, 81, 81, 81, 82, 82, 79, 79, 79, 79, 79, 79, 79, 79, 79, 79, 79, 79, 80, 80, 80, 3, 3, 78, 78, 36, 11, 11, 65, 65, 13, 63, 64, 64, 46, 46, 62, 62, 83, 83, 10, 10, 10, 10, 61, 61, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 58, 52, 52, 52, 56, 56, 56, 56, 54, 54, 53, 53, 55, 55, 57, 57, 57, 57, 57, 57, 69, 69, 69, 85, 4, 4, 4, 51, 51, 51, 51, 45, 45, 6, 6, 47, 47, 76, 76, 76, 84, 86, 35, 8, 8, 60, 60, 59, 59, 31, 74, 74, 73, 72, 72, 71, 71, 71, 71, 71, 71, 71, 71, 71, 34, 34, 34, 34, 34, 34, 34, 34, 34, 34, 34, 34, 34, 34, 34, 34, 34, 34, 34, 34, 34, 7, 7, 14, 14, 32, 32, 32, 87, 87, 50, 50, 50, 50, 29, 29, 29, 30, 30, 15, 15, 15, 28, 28, 28, 28, 28, 28, 28, 28, 2, 2, 33, 33, 33, 16, 16, 16, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 27, 26, 26, 25, 20, 24, 22, 18, 21, 17, 19, 23, 0];
const ARITY = [1, 3, 3, 3, 2, 1, 1, 1, 1, 4, 5, 3, 4, 1, 2, 2, 3, 7, 1, 2, 1, 5, 5, 5, 5, 2, 3, 1, 3, 1, 1, 5, 3, 3, 3, 3, 3, 1, 1, 1, 3, 3, 5, 3, 3, 6, 1, 1, 1, 1, 1, 1, 5, 7, 6, 7, 7, 7, 6, 8, 7, 6, 1, 7, 3, 8, 12, 4, 8, 4, 7, 9, 6, 6, 7, 6, 1, 7, 7, 3, 1, 1, 1, 2, 3, 1, 2, 3, 3, 3, 1, 1, 3, 1, 3, 1, 3, 1, 2, 2, 3, 3, 4, 1, 3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3, 3, 1, 3, 3, 3, 1, 3, 3, 1, 1, 5, 3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3, 3, 1, 3, 3, 3, 1, 3, 3, 1, 1, 5, 4, 4, 4, 1, 1, 1, 2, 3, 2, 3, 3, 5, 5, 1, 3, 3, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3, 1, 1, 1, 1, 3, 1, 2, 1, 1, 1, 1, 2, 1, 1, 1, 2, 1, 7, 2, 1, 1, 1, 1, 4, 9, 7, 5, 1, 1, 1, 1, 1, 1, 4, 1, 9, 15, 5, 7, 1, 8, 8, 8, 9, 9, 9, 10, 5, 7, 2, 2, 1, 1, 4, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2];
const GOTO = [{ "9": 7, "44": 5, "48": 1, "70": 2, "75": 8, "79": 10, "80": 18 }, {}, {}, {}, { "5": 52, "8": 43, "10": 29, "11": 33, "35": 46, "46": 26, "52": 36, "53": 39, "54": 38, "55": 40, "56": 37, "57": 41, "58": 32, "60": 47, "62": 27, "66": 24, "76": 44, "77": 51, "78": 34, "83": 28, "84": 45 }, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, { "63": 70, "64": 69 }, { "5": 52, "8": 43, "35": 46, "52": 36, "53": 39, "54": 38, "55": 40, "56": 37, "57": 41, "58": 72, "60": 47, "76": 44, "77": 51, "84": 45 }, {}, { "79": 75, "80": 18 }, {}, {}, { "1": 129, "2": 123, "4": 79, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 81, "47": 84, "50": 139, "51": 80, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "85": 78, "86": 87 }, { "49": 152 }, {}, {}, {}, {}, {}, {}, { "5": 52, "8": 43, "10": 161, "11": 33, "35": 46, "52": 36, "53": 39, "54": 38, "55": 40, "56": 37, "57": 41, "58": 32, "60": 47, "76": 44, "77": 51, "78": 34, "84": 45 }, { "12": 163 }, { "12": 180 }, { "13": 182 }, { "36": 184 }, {}, {}, {}, {}, {}, {}, {}, { "1": 129, "2": 123, "4": 79, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 81, "47": 84, "50": 139, "51": 80, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "85": 193, "86": 87 }, {}, {}, {}, {}, {}, { "59": 195 }, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, { "1": 129, "2": 123, "4": 79, "5": 52, "6": 82, "7": 201, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 81, "47": 84, "50": 139, "51": 80, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "85": 202, "86": 87 }, { "63": 70, "64": 204 }, { "65": 205 }, { "1": 129, "2": 123, "4": 79, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 81, "47": 84, "50": 139, "51": 80, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "85": 207, "86": 87 }, { "3": 208, "11": 210, "78": 34 }, { "3": 212, "11": 213, "78": 34, "81": 211, "82": 219 }, {}, {}, { "11": 223, "78": 34, "81": 222, "82": 219 }, { "79": 224, "80": 18 }, {}, {}, {}, {}, {}, { "65": 228 }, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, { "87": 243 }, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, { "50": 250 }, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, { "67": 251, "68": 253 }, {}, { "67": 257, "68": 253 }, { "5": 52, "8": 43, "10": 29, "11": 33, "35": 46, "52": 36, "53": 39, "54": 38, "55": 40, "56": 37, "57": 41, "58": 32, "60": 47, "62": 258, "76": 44, "77": 51, "78": 34, "83": 28, "84": 45 }, { "5": 52, "8": 43, "10": 29, "11": 33, "35": 46, "52": 36, "53": 39, "54": 38, "55": 40, "56": 37, "57": 41, "58": 32, "60": 47, "76": 44, "77": 51, "78": 34, "83": 259, "84": 45 }, {}, {}, { "1": 129, "2": 123, "4": 79, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 81, "47": 84, "50": 139, "51": 80, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "85": 260, "86": 87 }, { "5": 52, "8": 43, "35": 46, "55": 261, "57": 41, "60": 47, "76": 44, "77": 51, "84": 45 }, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, { "1": 129, "2": 123, "4": 79, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 81, "47": 84, "50": 139, "51": 80, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "85": 262, "86": 87 }, { "5": 52, "8": 43, "10": 29, "11": 33, "35": 46, "46": 263, "52": 36, "53": 39, "54": 38, "55": 40, "56": 37, "57": 41, "58": 32, "60": 47, "62": 27, "76": 44, "77": 51, "78": 34, "83": 28, "84": 45 }, {}, {}, {}, {}, { "5": 52, "8": 43, "35": 46, "53": 39, "54": 38, "55": 40, "56": 266, "57": 41, "60": 47, "76": 44, "77": 51, "84": 45 }, { "5": 52, "8": 43, "35": 46, "53": 39, "54": 38, "55": 40, "56": 267, "57": 41, "60": 47, "76": 44, "77": 51, "84": 45 }, { "5": 52, "8": 43, "35": 46, "53": 39, "54": 268, "55": 40, "57": 41, "60": 47, "76": 44, "77": 51, "84": 45 }, { "5": 52, "8": 43, "35": 46, "53": 39, "54": 269, "55": 40, "57": 41, "60": 47, "76": 44, "77": 51, "84": 45 }, { "5": 52, "8": 43, "35": 46, "53": 39, "54": 270, "55": 40, "57": 41, "60": 47, "76": 44, "77": 51, "84": 45 }, { "5": 52, "8": 43, "35": 46, "53": 271, "55": 40, "57": 41, "60": 47, "76": 44, "77": 51, "84": 45 }, {}, {}, {}, {}, {}, { "1": 129, "2": 123, "4": 79, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 81, "47": 84, "50": 139, "51": 80, "57": 86, "60": 47, "61": 277, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "85": 278, "86": 87 }, { "1": 129, "2": 123, "4": 79, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 81, "47": 84, "50": 139, "51": 80, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "85": 279, "86": 87 }, { "1": 129, "2": 123, "4": 79, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 81, "47": 84, "50": 139, "51": 80, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "85": 280, "86": 87 }, {}, {}, {}, { "37": 284, "38": 283, "75": 285, "79": 10, "80": 18 }, {}, {}, {}, {}, {}, {}, { "13": 182 }, {}, {}, { "13": 182 }, { "11": 223, "78": 34, "81": 290, "82": 219 }, { "5": 291 }, {}, { "63": 70, "64": 294 }, { "5": 52, "8": 43, "35": 46, "52": 36, "53": 39, "54": 38, "55": 40, "56": 37, "57": 41, "58": 295, "60": 47, "76": 44, "77": 51, "84": 45 }, {}, { "1": 129, "2": 123, "4": 79, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 81, "47": 84, "50": 139, "51": 80, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "85": 297, "86": 87 }, { "1": 129, "2": 123, "4": 79, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 81, "47": 84, "50": 139, "51": 80, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "85": 298, "86": 87 }, {}, { "13": 182 }, {}, {}, { "63": 302 }, {}, {}, {}, {}, { "1": 129, "2": 123, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 81, "47": 84, "50": 139, "51": 307, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "86": 87 }, { "1": 129, "2": 123, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 81, "47": 84, "50": 139, "51": 308, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "86": 87 }, { "1": 129, "2": 123, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 309, "47": 84, "50": 139, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "86": 87 }, { "1": 129, "2": 123, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 310, "47": 84, "50": 139, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "86": 87 }, { "1": 129, "2": 123, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 311, "47": 84, "50": 139, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "86": 87 }, { "1": 129, "2": 123, "5": 52, "6": 312, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "47": 84, "50": 139, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "86": 87 }, { "1": 129, "2": 123, "5": 52, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "47": 313, "50": 139, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "86": 87 }, {}, {}, {}, {}, {}, {}, {}, {}, {}, { "87": 321 }, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, { "1": 129, "2": 123, "4": 79, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 81, "47": 84, "50": 139, "51": 80, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "85": 326, "86": 87 }, { "1": 129, "2": 123, "4": 79, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 81, "47": 84, "50": 139, "51": 80, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "85": 327, "86": 87 }, {}, {}, {}, {}, {}, {}, { "5": 52, "8": 43, "10": 29, "11": 33, "35": 46, "46": 328, "52": 36, "53": 39, "54": 38, "55": 40, "56": 37, "57": 41, "58": 32, "60": 47, "62": 27, "76": 44, "77": 51, "78": 34, "83": 28, "84": 45 }, {}, {}, {}, { "1": 129, "2": 123, "4": 79, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 81, "47": 84, "50": 139, "51": 80, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "85": 330, "86": 87 }, {}, {}, {}, {}, {}, { "1": 129, "2": 123, "4": 79, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 81, "47": 84, "50": 139, "51": 80, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "85": 335, "86": 87 }, { "37": 336, "75": 285, "79": 10, "80": 18 }, {}, { "39": 341, "40": 338, "41": 340 }, { "37": 284, "38": 342, "75": 285, "79": 10, "80": 18 }, { "63": 343 }, { "75": 345, "79": 10, "80": 18 }, { "3": 346, "11": 210, "78": 34 }, {}, {}, {}, {}, {}, {}, { "65": 355 }, {}, {}, {}, {}, {}, {}, { "79": 361, "80": 18 }, {}, {}, { "11": 223, "78": 34, "81": 364, "82": 219 }, {}, {}, {}, {}, {}, {}, {}, { "5": 52, "8": 43, "10": 29, "11": 33, "35": 46, "46": 365, "52": 36, "53": 39, "54": 38, "55": 40, "56": 37, "57": 41, "58": 32, "60": 47, "62": 27, "76": 44, "77": 51, "78": 34, "83": 28, "84": 45 }, { "73": 367, "74": 366 }, { "30": 370, "50": 374 }, { "30": 375, "32": 376, "50": 374 }, { "50": 378 }, {}, {}, {}, { "68": 381 }, { "68": 383 }, { "5": 52, "8": 43, "10": 29, "11": 33, "35": 46, "46": 384, "52": 36, "53": 39, "54": 38, "55": 40, "56": 37, "57": 41, "58": 32, "60": 47, "62": 27, "76": 44, "77": 51, "78": 34, "83": 28, "84": 45 }, {}, {}, {}, {}, {}, {}, {}, { "1": 129, "2": 123, "4": 79, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 81, "47": 84, "50": 139, "51": 80, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "85": 387, "86": 87 }, {}, {}, {}, {}, {}, {}, {}, { "39": 394 }, {}, { "37": 336, "75": 285, "79": 10, "80": 18 }, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, { "5": 52, "8": 43, "10": 29, "11": 33, "35": 46, "46": 409, "52": 36, "53": 39, "54": 38, "55": 40, "56": 37, "57": 41, "58": 32, "60": 47, "62": 27, "76": 44, "77": 51, "78": 34, "83": 28, "84": 45 }, { "79": 411, "80": 18 }, {}, { "11": 223, "78": 34, "81": 413, "82": 219 }, { "11": 223, "78": 34, "81": 414, "82": 219 }, {}, {}, {}, {}, { "71": 420, "72": 419 }, {}, {}, { "87": 431 }, { "30": 432, "50": 374 }, {}, {}, {}, {}, { "87": 438 }, {}, {}, {}, {}, {}, {}, {}, {}, { "1": 129, "2": 123, "4": 79, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 81, "47": 84, "50": 139, "51": 80, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "85": 440, "86": 87 }, {}, {}, {}, {}, {}, {}, {}, {}, {}, { "5": 52, "8": 43, "10": 29, "11": 33, "35": 46, "46": 446, "52": 36, "53": 39, "54": 38, "55": 40, "56": 37, "57": 41, "58": 32, "60": 47, "62": 27, "76": 44, "77": 51, "78": 34, "83": 28, "84": 45 }, { "1": 129, "2": 123, "4": 79, "5": 52, "6": 82, "7": 447, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 81, "47": 84, "50": 139, "51": 80, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "85": 448, "86": 87 }, {}, {}, { "11": 223, "78": 34, "81": 451, "82": 219 }, {}, {}, {}, {}, { "11": 223, "78": 34, "81": 456, "82": 219 }, {}, {}, { "63": 459 }, {}, {}, {}, {}, {}, {}, {}, {}, {}, { "73": 464 }, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, { "30": 475, "50": 374 }, {}, {}, {}, { "30": 479, "50": 374 }, {}, { "2": 480 }, { "30": 481, "50": 374 }, {}, {}, {}, {}, {}, { "42": 485 }, { "1": 129, "2": 123, "4": 79, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 81, "47": 84, "50": 139, "51": 80, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "85": 487, "86": 87 }, {}, { "1": 129, "2": 123, "4": 79, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 81, "47": 84, "50": 139, "51": 80, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "85": 489, "86": 87 }, {}, {}, {}, { "11": 223, "78": 34, "81": 492, "82": 219 }, { "11": 223, "78": 34, "81": 493, "82": 219 }, {}, { "11": 223, "78": 34, "81": 495, "82": 219 }, { "11": 223, "78": 34, "81": 496, "82": 219 }, { "11": 223, "78": 34, "81": 497, "82": 219 }, { "11": 223, "78": 34, "81": 498, "82": 219 }, {}, { "1": 129, "2": 123, "4": 79, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 81, "47": 84, "50": 139, "51": 80, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "85": 500, "86": 87 }, { "1": 129, "2": 123, "4": 79, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 81, "47": 84, "50": 139, "51": 80, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "85": 501, "86": 87 }, {}, { "63": 70, "64": 502 }, {}, {}, {}, {}, {}, { "71": 503 }, { "1": 504 }, { "1": 506 }, { "2": 507 }, { "2": 508 }, { "50": 509 }, { "29": 510, "50": 511 }, {}, {}, {}, {}, {}, { "30": 519, "50": 374 }, {}, {}, {}, {}, {}, {}, {}, { "43": 528, "60": 529 }, {}, {}, {}, {}, { "1": 129, "2": 123, "4": 79, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 81, "47": 84, "50": 139, "51": 80, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "85": 533, "86": 87 }, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, { "30": 543, "50": 374 }, {}, {}, {}, {}, { "2": 549 }, {}, { "32": 550 }, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, { "2": 561 }, {}, {}, {}, {}, {}, { "60": 564 }, {}, {}, { "30": 370, "50": 374 }, {}, { "30": 566, "50": 374 }, {}, {}, {}, {}, {}, {}, { "1": 129, "2": 123, "4": 79, "5": 52, "6": 82, "8": 43, "14": 89, "15": 93, "16": 95, "17": 104, "18": 101, "19": 105, "20": 99, "21": 102, "22": 100, "23": 106, "24": 103, "25": 98, "26": 97, "27": 107, "28": 94, "29": 91, "30": 92, "31": 108, "32": 90, "33": 96, "34": 88, "35": 46, "45": 81, "47": 84, "50": 139, "51": 80, "57": 86, "60": 47, "63": 151, "69": 85, "76": 44, "77": 51, "84": 45, "85": 568, "86": 87 }, {}, {}, {}, { "30": 570, "50": 374 }, {}, { "30": 572, "50": 374 }, {}, {}];
const PARSER_ACTION = [{ "6": [1, 6], "8": [1, 19], "61": [1, 16], "67": [1, 13], "72": [1, 3], "74": [1, 17], "82": [1, 15], "89": [1, 9], "98": [1, 20], "105": [1, 14], "110": [1, 11], "114": [1, 4], "127": [1, 12] }, { "1": [0, 0] }, { "1": [2, 0] }, { "65": [1, 22], "73": [1, 23], "118": [1, 21] }, { "8": [1, 42], "28": [1, 49], "39": [1, 35], "48": [1, 31], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "103": [1, 55], "104": [1, 54], "108": [1, 30], "123": [1, 56], "128": [1, 25], "129": [1, 50], "130": [1, 48] }, { "1": [2, 5] }, { "39": [1, 60] }, { "1": [2, 29] }, { "1": [2, 30], "112": [1, 61] }, { "21": [1, 62] }, { "23": [1, 63] }, { "23": [1, 64] }, { "69": [1, 65] }, { "124": [1, 66] }, { "8": [1, 67] }, { "8": [1, 68] }, { "48": [1, 71] }, { "8": [1, 42], "28": [1, 49], "48": [1, 73], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "103": [1, 55], "104": [1, 54], "123": [1, 56], "129": [1, 50], "130": [1, 48] }, { "9": [2, 76], "23": [2, 76], "112": [1, 74] }, { "8": [1, 19], "61": [1, 16], "67": [1, 13], "74": [1, 17], "82": [1, 15], "105": [1, 14], "127": [1, 12] }, { "48": [1, 76] }, { "55": [1, 77] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "16": [1, 153], "17": [1, 154], "42": [1, 155] }, { "1": [2, 4] }, { "18": [1, 156], "48": [1, 157] }, { "18": [1, 158], "64": [1, 159] }, { "1": [2, 93], "9": [2, 93], "18": [2, 93], "23": [2, 93], "64": [2, 93], "113": [1, 160], "131": [2, 93] }, { "1": [2, 95], "9": [2, 95], "18": [2, 95], "23": [2, 95], "64": [2, 95], "113": [2, 95], "131": [2, 95] }, { "1": [2, 97], "9": [2, 97], "18": [2, 97], "23": [2, 97], "64": [2, 97], "113": [2, 97], "131": [2, 97] }, { "8": [1, 42], "28": [1, 49], "39": [1, 35], "48": [1, 31], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "103": [1, 55], "104": [1, 54], "123": [1, 56], "128": [1, 162], "129": [1, 50], "130": [1, 48] }, { "20": [1, 167], "22": [1, 165], "24": [1, 168], "25": [1, 166], "76": [1, 174], "77": [1, 175], "86": [1, 171], "94": [1, 177], "95": [1, 178], "111": [1, 164], "115": [1, 172], "120": [1, 170], "122": [1, 173], "132": [1, 169], "133": [1, 176], "134": [1, 179] }, { "20": [1, 167], "22": [1, 165], "24": [1, 168], "25": [1, 166], "76": [1, 174], "77": [1, 175], "86": [1, 171], "94": [1, 177], "95": [1, 178], "115": [1, 172], "120": [1, 170], "122": [1, 173], "132": [1, 169], "133": [1, 176], "134": [1, 179] }, { "48": [1, 183], "130": [1, 181] }, { "1": [2, 85], "5": [2, 85], "9": [2, 85], "19": [2, 85], "23": [2, 85], "26": [1, 185], "48": [2, 85], "112": [2, 85], "130": [2, 85] }, { "1": [2, 82], "5": [2, 82], "9": [2, 82], "19": [2, 82], "23": [2, 82], "26": [2, 82], "48": [2, 82], "112": [2, 82], "130": [2, 82] }, { "12": [1, 186], "14": [1, 187], "20": [2, 120], "22": [2, 120], "24": [2, 120], "25": [2, 120], "76": [2, 120], "77": [2, 120], "86": [2, 120], "94": [2, 120], "95": [2, 120], "111": [2, 120], "115": [2, 120], "120": [2, 120], "122": [2, 120], "132": [2, 120], "133": [2, 120], "134": [2, 120] }, { "7": [1, 190], "10": [1, 188], "12": [2, 121], "14": [2, 121], "15": [1, 189], "20": [2, 121], "22": [2, 121], "24": [2, 121], "25": [2, 121], "76": [2, 121], "77": [2, 121], "86": [2, 121], "94": [2, 121], "95": [2, 121], "111": [2, 121], "115": [2, 121], "120": [2, 121], "122": [2, 121], "132": [2, 121], "133": [2, 121], "134": [2, 121] }, { "7": [2, 124], "10": [2, 124], "11": [1, 191], "12": [2, 124], "14": [2, 124], "15": [2, 124], "20": [2, 124], "22": [2, 124], "24": [2, 124], "25": [2, 124], "76": [2, 124], "77": [2, 124], "86": [2, 124], "94": [2, 124], "95": [2, 124], "111": [2, 124], "115": [2, 124], "120": [2, 124], "122": [2, 124], "132": [2, 124], "133": [2, 124], "134": [2, 124] }, { "7": [2, 128], "10": [2, 128], "11": [2, 128], "12": [2, 128], "14": [2, 128], "15": [2, 128], "20": [2, 128], "22": [2, 128], "24": [2, 128], "25": [2, 128], "76": [2, 128], "77": [2, 128], "86": [2, 128], "94": [2, 128], "95": [2, 128], "111": [2, 128], "115": [2, 128], "120": [2, 128], "122": [2, 128], "132": [2, 128], "133": [2, 128], "134": [2, 128] }, { "7": [2, 131], "10": [2, 131], "11": [2, 131], "12": [2, 131], "14": [2, 131], "15": [2, 131], "20": [2, 131], "22": [2, 131], "24": [2, 131], "25": [2, 131], "76": [2, 131], "77": [2, 131], "86": [2, 131], "94": [2, 131], "95": [2, 131], "111": [2, 131], "115": [2, 131], "120": [2, 131], "122": [2, 131], "132": [2, 131], "133": [2, 131], "134": [2, 131] }, { "7": [2, 132], "10": [2, 132], "11": [2, 132], "12": [2, 132], "14": [2, 132], "15": [2, 132], "20": [2, 132], "22": [2, 132], "24": [2, 132], "25": [2, 132], "76": [2, 132], "77": [2, 132], "86": [2, 132], "92": [1, 192], "94": [2, 132], "95": [2, 132], "111": [2, 132], "115": [2, 132], "120": [2, 132], "122": [2, 132], "132": [2, 132], "133": [2, 132], "134": [2, 132] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "1": [2, 135], "5": [2, 135], "7": [2, 135], "9": [2, 135], "10": [2, 135], "11": [2, 135], "12": [2, 135], "13": [2, 135], "14": [2, 135], "15": [2, 135], "18": [2, 135], "19": [2, 135], "20": [2, 135], "22": [2, 135], "23": [2, 135], "24": [2, 135], "25": [2, 135], "26": [2, 135], "48": [2, 135], "62": [2, 135], "64": [2, 135], "76": [2, 135], "77": [2, 135], "86": [2, 135], "92": [2, 135], "94": [2, 135], "95": [2, 135], "111": [2, 135], "112": [2, 135], "113": [2, 135], "115": [2, 135], "120": [2, 135], "122": [2, 135], "130": [2, 135], "131": [2, 135], "132": [2, 135], "133": [2, 135], "134": [2, 135] }, { "1": [2, 136], "5": [2, 136], "7": [2, 136], "9": [2, 136], "10": [2, 136], "11": [2, 136], "12": [2, 136], "13": [2, 136], "14": [2, 136], "15": [2, 136], "18": [2, 136], "19": [2, 136], "20": [2, 136], "22": [2, 136], "23": [2, 136], "24": [2, 136], "25": [2, 136], "26": [2, 136], "48": [2, 136], "62": [2, 136], "64": [2, 136], "76": [2, 136], "77": [2, 136], "86": [2, 136], "92": [2, 136], "94": [2, 136], "95": [2, 136], "111": [2, 136], "112": [2, 136], "113": [2, 136], "115": [2, 136], "120": [2, 136], "122": [2, 136], "130": [2, 136], "131": [2, 136], "132": [2, 136], "133": [2, 136], "134": [2, 136] }, { "1": [2, 137], "5": [2, 137], "7": [2, 137], "9": [2, 137], "10": [2, 137], "11": [2, 137], "12": [2, 137], "13": [2, 137], "14": [2, 137], "15": [2, 137], "18": [2, 137], "19": [2, 137], "20": [2, 137], "22": [2, 137], "23": [2, 137], "24": [2, 137], "25": [2, 137], "26": [2, 137], "48": [2, 137], "62": [2, 137], "64": [2, 137], "76": [2, 137], "77": [2, 137], "86": [2, 137], "92": [2, 137], "94": [2, 137], "95": [2, 137], "111": [2, 137], "112": [2, 137], "113": [2, 137], "115": [2, 137], "120": [2, 137], "122": [2, 137], "130": [2, 137], "131": [2, 137], "132": [2, 137], "133": [2, 137], "134": [2, 137] }, { "1": [2, 138], "5": [2, 138], "7": [2, 138], "9": [2, 138], "10": [2, 138], "11": [2, 138], "12": [2, 138], "13": [2, 138], "14": [2, 138], "15": [2, 138], "18": [2, 138], "19": [2, 138], "20": [2, 138], "22": [2, 138], "23": [2, 138], "24": [2, 138], "25": [2, 138], "26": [2, 138], "48": [2, 138], "62": [2, 138], "64": [2, 138], "76": [2, 138], "77": [2, 138], "86": [2, 138], "92": [2, 138], "94": [2, 138], "95": [2, 138], "111": [2, 138], "112": [2, 138], "113": [2, 138], "115": [2, 138], "120": [2, 138], "122": [2, 138], "130": [2, 138], "131": [2, 138], "132": [2, 138], "133": [2, 138], "134": [2, 138] }, { "1": [2, 139], "5": [2, 139], "7": [2, 139], "9": [2, 139], "10": [2, 139], "11": [2, 139], "12": [2, 139], "13": [2, 139], "14": [2, 139], "15": [2, 139], "18": [2, 139], "19": [2, 139], "20": [2, 139], "22": [2, 139], "23": [2, 139], "24": [2, 139], "25": [2, 139], "26": [2, 139], "48": [2, 139], "62": [2, 139], "64": [2, 139], "76": [2, 139], "77": [2, 139], "86": [2, 139], "92": [2, 139], "94": [2, 139], "95": [2, 139], "111": [2, 139], "112": [2, 139], "113": [2, 139], "115": [2, 139], "120": [2, 139], "122": [2, 139], "130": [2, 139], "131": [2, 139], "132": [2, 139], "133": [2, 139], "134": [2, 139] }, { "48": [1, 196], "131": [1, 194] }, { "1": [2, 162], "5": [2, 162], "7": [2, 162], "9": [2, 162], "10": [2, 162], "11": [2, 162], "12": [2, 162], "13": [2, 162], "14": [2, 162], "15": [2, 162], "18": [2, 162], "19": [2, 162], "20": [2, 162], "22": [2, 162], "23": [2, 162], "24": [2, 162], "25": [2, 162], "26": [2, 162], "48": [2, 162], "62": [2, 162], "64": [2, 162], "76": [2, 162], "77": [2, 162], "86": [2, 162], "92": [2, 162], "94": [2, 162], "95": [2, 162], "111": [2, 162], "112": [2, 162], "113": [2, 162], "115": [2, 162], "120": [2, 162], "122": [2, 162], "130": [2, 162], "131": [2, 162], "132": [2, 162], "133": [2, 162], "134": [2, 162] }, { "1": [2, 160], "5": [2, 160], "7": [2, 160], "9": [2, 160], "10": [2, 160], "11": [2, 160], "12": [2, 160], "13": [2, 160], "14": [2, 160], "15": [2, 160], "18": [2, 160], "19": [2, 160], "20": [2, 160], "22": [2, 160], "23": [2, 160], "24": [2, 160], "25": [2, 160], "26": [2, 160], "48": [2, 160], "62": [2, 160], "64": [2, 160], "76": [2, 160], "77": [2, 160], "86": [2, 160], "92": [2, 160], "94": [2, 160], "95": [2, 160], "111": [2, 160], "112": [2, 160], "113": [2, 160], "115": [2, 160], "120": [2, 160], "122": [2, 160], "130": [2, 160], "131": [2, 160], "132": [2, 160], "133": [2, 160], "134": [2, 160] }, { "8": [1, 197] }, { "8": [1, 198] }, { "8": [1, 199] }, { "8": [2, 46], "48": [2, 46] }, { "8": [2, 47], "48": [2, 47] }, { "8": [2, 48], "48": [2, 48] }, { "8": [2, 49], "48": [2, 49] }, { "8": [2, 50] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "62": [1, 200], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "19": [1, 203], "48": [1, 71] }, { "48": [1, 206] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "39": [1, 35], "109": [1, 209] }, { "8": [1, 214], "39": [1, 35], "61": [1, 217], "63": [1, 215], "74": [1, 218], "109": [1, 209], "117": [1, 216] }, { "21": [1, 220] }, { "21": [1, 221] }, { "8": [1, 214], "39": [1, 35], "61": [1, 217], "63": [1, 215], "74": [1, 218], "117": [1, 216] }, { "8": [1, 19], "61": [1, 16], "67": [1, 13], "74": [1, 17], "82": [1, 15], "105": [1, 14], "127": [1, 12] }, { "13": [1, 226], "62": [1, 225] }, { "13": [2, 91], "19": [2, 91], "62": [2, 91] }, { "1": [2, 90], "5": [2, 90], "7": [2, 90], "9": [2, 90], "10": [2, 90], "11": [2, 90], "12": [2, 90], "13": [2, 90], "14": [2, 90], "15": [2, 90], "18": [2, 90], "19": [2, 90], "23": [2, 90], "26": [2, 90], "48": [2, 90], "62": [2, 90], "64": [2, 90], "92": [2, 90], "112": [2, 90], "113": [2, 90], "130": [2, 90], "131": [2, 90] }, { "111": [1, 227] }, { "111": [1, 164] }, { "48": [1, 206] }, { "9": [1, 229] }, { "21": [1, 230] }, { "1": [2, 1] }, { "1": [2, 2] }, { "1": [2, 143], "5": [2, 143], "9": [2, 143], "12": [1, 231], "13": [2, 143], "14": [1, 232], "18": [2, 143], "19": [2, 143], "23": [2, 143], "26": [2, 143], "48": [2, 143], "62": [2, 143], "64": [2, 143], "112": [2, 143], "113": [2, 143], "130": [2, 143], "131": [2, 143] }, { "1": [2, 144], "5": [2, 144], "7": [1, 235], "9": [2, 144], "10": [1, 233], "12": [2, 144], "13": [2, 144], "14": [2, 144], "15": [1, 234], "18": [2, 144], "19": [2, 144], "23": [2, 144], "26": [2, 144], "48": [2, 144], "62": [2, 144], "64": [2, 144], "112": [2, 144], "113": [2, 144], "130": [2, 144], "131": [2, 144] }, { "1": [2, 147], "5": [2, 147], "7": [2, 147], "9": [2, 147], "10": [2, 147], "11": [1, 236], "12": [2, 147], "13": [2, 147], "14": [2, 147], "15": [2, 147], "18": [2, 147], "19": [2, 147], "23": [2, 147], "26": [2, 147], "48": [2, 147], "62": [2, 147], "64": [2, 147], "112": [2, 147], "113": [2, 147], "130": [2, 147], "131": [2, 147] }, { "1": [2, 151], "5": [2, 151], "7": [2, 151], "9": [2, 151], "10": [2, 151], "11": [2, 151], "12": [2, 151], "13": [2, 151], "14": [2, 151], "15": [2, 151], "18": [2, 151], "19": [2, 151], "23": [2, 151], "26": [2, 151], "48": [2, 151], "62": [2, 151], "64": [2, 151], "112": [2, 151], "113": [2, 151], "130": [2, 151], "131": [2, 151] }, { "1": [2, 90], "5": [2, 90], "7": [2, 90], "9": [2, 90], "10": [2, 90], "11": [2, 90], "12": [2, 90], "13": [2, 90], "14": [2, 90], "15": [2, 90], "18": [2, 90], "19": [2, 90], "23": [2, 90], "26": [2, 90], "48": [2, 90], "62": [2, 90], "64": [2, 90], "92": [2, 90], "111": [1, 237], "112": [2, 90], "113": [2, 90], "130": [2, 90], "131": [2, 90] }, { "1": [2, 154], "5": [2, 154], "7": [2, 154], "9": [2, 154], "10": [2, 154], "11": [2, 154], "12": [2, 154], "13": [2, 154], "14": [2, 154], "15": [2, 154], "18": [2, 154], "19": [2, 154], "23": [2, 154], "26": [2, 154], "48": [2, 154], "62": [2, 154], "64": [2, 154], "112": [2, 154], "113": [2, 154], "130": [2, 154], "131": [2, 154] }, { "1": [2, 155], "5": [2, 155], "7": [2, 155], "9": [2, 155], "10": [2, 155], "11": [2, 155], "12": [2, 155], "13": [2, 155], "14": [2, 155], "15": [2, 155], "18": [2, 155], "19": [2, 155], "23": [2, 155], "26": [2, 155], "48": [2, 155], "62": [2, 155], "64": [2, 155], "92": [1, 238], "112": [2, 155], "113": [2, 155], "130": [2, 155], "131": [2, 155] }, { "1": [2, 140], "5": [2, 140], "7": [2, 140], "9": [2, 140], "10": [2, 140], "11": [2, 140], "12": [2, 140], "13": [2, 140], "14": [2, 140], "15": [2, 140], "18": [2, 140], "19": [2, 140], "23": [2, 140], "26": [2, 140], "48": [2, 140], "62": [2, 140], "64": [2, 140], "92": [2, 140], "112": [2, 140], "113": [2, 140], "130": [2, 140], "131": [2, 140] }, { "1": [2, 141], "5": [2, 141], "7": [2, 141], "9": [2, 141], "10": [2, 141], "11": [2, 141], "12": [2, 141], "13": [2, 141], "14": [2, 141], "15": [2, 141], "18": [2, 141], "19": [2, 141], "23": [2, 141], "26": [2, 141], "48": [2, 141], "62": [2, 141], "64": [2, 141], "92": [2, 141], "112": [2, 141], "113": [2, 141], "130": [2, 141], "131": [2, 141] }, { "1": [2, 142], "5": [2, 142], "7": [2, 142], "9": [2, 142], "10": [2, 142], "11": [2, 142], "12": [2, 142], "13": [2, 142], "14": [2, 142], "15": [2, 142], "18": [2, 142], "19": [2, 142], "23": [2, 142], "26": [2, 142], "48": [2, 142], "62": [2, 142], "64": [2, 142], "92": [2, 142], "112": [2, 142], "113": [2, 142], "130": [2, 142], "131": [2, 142] }, { "1": [2, 184], "5": [2, 184], "7": [2, 184], "9": [2, 184], "10": [2, 184], "11": [2, 184], "12": [2, 184], "13": [2, 184], "14": [2, 184], "15": [2, 184], "18": [2, 184], "19": [2, 184], "23": [2, 184], "26": [2, 184], "48": [2, 184], "62": [2, 184], "64": [2, 184], "92": [2, 184], "112": [2, 184], "113": [2, 184], "130": [2, 184], "131": [2, 184] }, { "1": [2, 185], "5": [2, 185], "7": [2, 185], "9": [2, 185], "10": [2, 185], "11": [2, 185], "12": [2, 185], "13": [2, 185], "14": [2, 185], "15": [2, 185], "18": [2, 185], "19": [2, 185], "23": [2, 185], "26": [2, 185], "48": [2, 185], "62": [2, 185], "64": [2, 185], "92": [2, 185], "112": [2, 185], "113": [2, 185], "130": [2, 185], "131": [2, 185] }, { "1": [2, 186], "5": [2, 186], "7": [2, 186], "9": [2, 186], "10": [2, 186], "11": [2, 186], "12": [2, 186], "13": [2, 186], "14": [2, 186], "15": [2, 186], "18": [2, 186], "19": [2, 186], "23": [2, 186], "26": [2, 186], "48": [2, 186], "62": [2, 186], "64": [2, 186], "92": [2, 186], "112": [2, 186], "113": [2, 186], "130": [2, 186], "131": [2, 186] }, { "1": [2, 187], "5": [2, 187], "7": [2, 187], "9": [2, 187], "10": [2, 187], "11": [2, 187], "12": [2, 187], "13": [2, 187], "14": [2, 187], "15": [2, 187], "18": [2, 187], "19": [2, 187], "23": [2, 187], "26": [2, 187], "48": [2, 187], "62": [2, 187], "64": [2, 187], "92": [2, 187], "112": [2, 187], "113": [2, 187], "130": [2, 187], "131": [2, 187] }, { "1": [2, 188], "5": [2, 188], "7": [2, 188], "9": [2, 188], "10": [2, 188], "11": [2, 188], "12": [2, 188], "13": [2, 188], "14": [2, 188], "15": [2, 188], "18": [2, 188], "19": [2, 188], "23": [2, 188], "26": [2, 188], "48": [2, 188], "62": [2, 188], "64": [2, 188], "92": [2, 188], "112": [2, 188], "113": [2, 188], "130": [2, 188], "131": [2, 188] }, { "1": [2, 189], "5": [2, 189], "7": [2, 189], "9": [2, 189], "10": [2, 189], "11": [2, 189], "12": [2, 189], "13": [2, 189], "14": [2, 189], "15": [2, 189], "18": [2, 189], "19": [2, 189], "23": [2, 189], "26": [2, 189], "48": [2, 189], "62": [2, 189], "64": [2, 189], "92": [2, 189], "112": [2, 189], "113": [2, 189], "130": [2, 189], "131": [2, 189] }, { "1": [2, 190], "5": [2, 190], "7": [2, 190], "9": [2, 190], "10": [2, 190], "11": [2, 190], "12": [2, 190], "13": [2, 190], "14": [2, 190], "15": [2, 190], "18": [2, 190], "19": [2, 190], "23": [2, 190], "26": [2, 190], "48": [2, 190], "62": [2, 190], "64": [2, 190], "92": [2, 190], "112": [2, 190], "113": [2, 190], "130": [2, 190], "131": [2, 190] }, { "1": [2, 191], "5": [2, 191], "7": [2, 191], "9": [2, 191], "10": [2, 191], "11": [2, 191], "12": [2, 191], "13": [2, 191], "14": [2, 191], "15": [2, 191], "18": [2, 191], "19": [2, 191], "23": [2, 191], "26": [2, 191], "48": [2, 191], "62": [2, 191], "64": [2, 191], "92": [2, 191], "112": [2, 191], "113": [2, 191], "130": [2, 191], "131": [2, 191] }, { "1": [2, 192], "5": [2, 192], "7": [2, 192], "9": [2, 192], "10": [2, 192], "11": [2, 192], "12": [2, 192], "13": [2, 192], "14": [2, 192], "15": [2, 192], "18": [2, 192], "19": [2, 192], "23": [2, 192], "26": [2, 192], "48": [2, 192], "62": [2, 192], "64": [2, 192], "92": [2, 192], "112": [2, 192], "113": [2, 192], "130": [2, 192], "131": [2, 192] }, { "1": [2, 193], "5": [2, 193], "7": [2, 193], "9": [2, 193], "10": [2, 193], "11": [2, 193], "12": [2, 193], "13": [2, 193], "14": [2, 193], "15": [2, 193], "18": [2, 193], "19": [2, 193], "23": [2, 193], "26": [2, 193], "48": [2, 193], "62": [2, 193], "64": [2, 193], "92": [2, 193], "112": [2, 193], "113": [2, 193], "130": [2, 193], "131": [2, 193] }, { "1": [2, 194], "5": [2, 194], "7": [2, 194], "9": [2, 194], "10": [2, 194], "11": [2, 194], "12": [2, 194], "13": [2, 194], "14": [2, 194], "15": [2, 194], "18": [2, 194], "19": [2, 194], "23": [2, 194], "26": [2, 194], "48": [2, 194], "62": [2, 194], "64": [2, 194], "92": [2, 194], "112": [2, 194], "113": [2, 194], "130": [2, 194], "131": [2, 194] }, { "1": [2, 195], "5": [2, 195], "7": [2, 195], "9": [2, 195], "10": [2, 195], "11": [2, 195], "12": [2, 195], "13": [2, 195], "14": [2, 195], "15": [2, 195], "18": [2, 195], "19": [2, 195], "23": [2, 195], "26": [2, 195], "48": [2, 195], "62": [2, 195], "64": [2, 195], "92": [2, 195], "112": [2, 195], "113": [2, 195], "130": [2, 195], "131": [2, 195] }, { "1": [2, 196], "5": [2, 196], "7": [2, 196], "9": [2, 196], "10": [2, 196], "11": [2, 196], "12": [2, 196], "13": [2, 196], "14": [2, 196], "15": [2, 196], "18": [2, 196], "19": [2, 196], "23": [2, 196], "26": [2, 196], "48": [2, 196], "62": [2, 196], "64": [2, 196], "92": [2, 196], "112": [2, 196], "113": [2, 196], "130": [2, 196], "131": [2, 196] }, { "1": [2, 197], "5": [2, 197], "7": [2, 197], "9": [2, 197], "10": [2, 197], "11": [2, 197], "12": [2, 197], "13": [2, 197], "14": [2, 197], "15": [2, 197], "18": [2, 197], "19": [2, 197], "23": [2, 197], "26": [2, 197], "48": [2, 197], "62": [2, 197], "64": [2, 197], "92": [2, 197], "112": [2, 197], "113": [2, 197], "130": [2, 197], "131": [2, 197] }, { "1": [2, 198], "5": [2, 198], "7": [2, 198], "9": [2, 198], "10": [2, 198], "11": [2, 198], "12": [2, 198], "13": [2, 198], "14": [2, 198], "15": [2, 198], "18": [2, 198], "19": [2, 198], "23": [2, 198], "26": [2, 198], "48": [2, 198], "62": [2, 198], "64": [2, 198], "92": [2, 198], "112": [2, 198], "113": [2, 198], "130": [2, 198], "131": [2, 198] }, { "1": [2, 199], "5": [2, 199], "7": [2, 199], "9": [2, 199], "10": [2, 199], "11": [2, 199], "12": [2, 199], "13": [2, 199], "14": [2, 199], "15": [2, 199], "18": [2, 199], "19": [2, 199], "23": [2, 199], "26": [2, 199], "48": [2, 199], "62": [2, 199], "64": [2, 199], "92": [2, 199], "112": [2, 199], "113": [2, 199], "130": [2, 199], "131": [2, 199] }, { "1": [2, 200], "5": [2, 200], "7": [2, 200], "9": [2, 200], "10": [2, 200], "11": [2, 200], "12": [2, 200], "13": [2, 200], "14": [2, 200], "15": [2, 200], "18": [2, 200], "19": [2, 200], "23": [2, 200], "26": [2, 200], "48": [2, 200], "62": [2, 200], "64": [2, 200], "92": [2, 200], "112": [2, 200], "113": [2, 200], "130": [2, 200], "131": [2, 200] }, { "1": [2, 201], "5": [2, 201], "7": [2, 201], "9": [2, 201], "10": [2, 201], "11": [2, 201], "12": [2, 201], "13": [2, 201], "14": [2, 201], "15": [2, 201], "18": [2, 201], "19": [2, 201], "23": [2, 201], "26": [2, 201], "48": [2, 201], "62": [2, 201], "64": [2, 201], "92": [2, 201], "112": [2, 201], "113": [2, 201], "130": [2, 201], "131": [2, 201] }, { "1": [2, 202], "5": [2, 202], "7": [2, 202], "9": [2, 202], "10": [2, 202], "11": [2, 202], "12": [2, 202], "13": [2, 202], "14": [2, 202], "15": [2, 202], "18": [2, 202], "19": [2, 202], "23": [2, 202], "26": [2, 202], "48": [2, 202], "62": [2, 202], "64": [2, 202], "92": [2, 202], "112": [2, 202], "113": [2, 202], "130": [2, 202], "131": [2, 202] }, { "1": [2, 203], "5": [2, 203], "7": [2, 203], "9": [2, 203], "10": [2, 203], "11": [2, 203], "12": [2, 203], "13": [2, 203], "14": [2, 203], "15": [2, 203], "18": [2, 203], "19": [2, 203], "23": [2, 203], "26": [2, 203], "48": [2, 203], "62": [2, 203], "64": [2, 203], "92": [2, 203], "112": [2, 203], "113": [2, 203], "130": [2, 203], "131": [2, 203] }, { "1": [2, 204], "5": [2, 204], "7": [2, 204], "9": [2, 204], "10": [2, 204], "11": [2, 204], "12": [2, 204], "13": [2, 204], "14": [2, 204], "15": [2, 204], "18": [2, 204], "19": [2, 204], "23": [2, 204], "26": [2, 204], "48": [2, 204], "62": [2, 204], "64": [2, 204], "92": [2, 204], "112": [2, 204], "113": [2, 204], "130": [2, 204], "131": [2, 204] }, { "31": [1, 242], "35": [1, 240], "45": [1, 241], "53": [1, 239] }, { "1": [2, 258], "5": [2, 258], "7": [2, 258], "9": [2, 258], "10": [2, 258], "11": [2, 258], "12": [2, 258], "13": [2, 258], "14": [2, 258], "15": [2, 258], "18": [2, 258], "19": [2, 258], "23": [2, 258], "26": [2, 258], "48": [2, 258], "62": [2, 258], "64": [2, 258], "92": [2, 258], "112": [2, 258], "113": [2, 258], "130": [2, 258], "131": [2, 258] }, { "1": [2, 269], "5": [2, 269], "7": [2, 269], "9": [2, 269], "10": [2, 269], "11": [2, 269], "12": [2, 269], "13": [2, 269], "14": [2, 269], "15": [2, 269], "18": [2, 269], "19": [2, 269], "23": [2, 269], "26": [2, 269], "48": [2, 269], "62": [2, 269], "64": [2, 269], "92": [2, 269], "112": [2, 269], "113": [2, 269], "130": [2, 269], "131": [2, 269] }, { "1": [2, 268], "5": [2, 268], "7": [2, 268], "9": [2, 268], "10": [2, 268], "11": [2, 268], "12": [2, 268], "13": [2, 268], "14": [2, 268], "15": [2, 268], "18": [2, 268], "19": [2, 268], "23": [2, 268], "26": [2, 268], "48": [2, 268], "62": [2, 268], "64": [2, 268], "92": [2, 268], "112": [2, 268], "113": [2, 268], "130": [2, 268], "131": [2, 268] }, { "1": [2, 267], "5": [2, 267], "7": [2, 267], "9": [2, 267], "10": [2, 267], "11": [2, 267], "12": [2, 267], "13": [2, 267], "14": [2, 267], "15": [2, 267], "18": [2, 267], "19": [2, 267], "23": [2, 267], "26": [2, 267], "48": [2, 267], "62": [2, 267], "64": [2, 267], "92": [2, 267], "112": [2, 267], "113": [2, 267], "130": [2, 267], "131": [2, 267] }, { "1": [2, 263], "5": [2, 263], "7": [2, 263], "9": [2, 263], "10": [2, 263], "11": [2, 263], "12": [2, 263], "13": [2, 263], "14": [2, 263], "15": [2, 263], "18": [2, 263], "19": [2, 263], "23": [2, 263], "26": [2, 263], "48": [2, 263], "62": [2, 263], "64": [2, 263], "92": [2, 263], "112": [2, 263], "113": [2, 263], "130": [2, 263], "131": [2, 263] }, { "1": [2, 266], "5": [2, 266], "7": [2, 266], "9": [2, 266], "10": [2, 266], "11": [2, 266], "12": [2, 266], "13": [2, 266], "14": [2, 266], "15": [2, 266], "18": [2, 266], "19": [2, 266], "23": [2, 266], "26": [2, 266], "48": [2, 266], "62": [2, 266], "64": [2, 266], "92": [2, 266], "112": [2, 266], "113": [2, 266], "130": [2, 266], "131": [2, 266] }, { "1": [2, 265], "5": [2, 265], "7": [2, 265], "9": [2, 265], "10": [2, 265], "11": [2, 265], "12": [2, 265], "13": [2, 265], "14": [2, 265], "15": [2, 265], "18": [2, 265], "19": [2, 265], "23": [2, 265], "26": [2, 265], "48": [2, 265], "62": [2, 265], "64": [2, 265], "92": [2, 265], "112": [2, 265], "113": [2, 265], "130": [2, 265], "131": [2, 265] }, { "1": [2, 264], "5": [2, 264], "7": [2, 264], "9": [2, 264], "10": [2, 264], "11": [2, 264], "12": [2, 264], "13": [2, 264], "14": [2, 264], "15": [2, 264], "18": [2, 264], "19": [2, 264], "23": [2, 264], "26": [2, 264], "48": [2, 264], "62": [2, 264], "64": [2, 264], "92": [2, 264], "112": [2, 264], "113": [2, 264], "130": [2, 264], "131": [2, 264] }, { "1": [2, 262], "5": [2, 262], "7": [2, 262], "9": [2, 262], "10": [2, 262], "11": [2, 262], "12": [2, 262], "13": [2, 262], "14": [2, 262], "15": [2, 262], "18": [2, 262], "19": [2, 262], "23": [2, 262], "26": [2, 262], "48": [2, 262], "62": [2, 262], "64": [2, 262], "92": [2, 262], "112": [2, 262], "113": [2, 262], "130": [2, 262], "131": [2, 262] }, { "1": [2, 261], "5": [2, 261], "7": [2, 261], "9": [2, 261], "10": [2, 261], "11": [2, 261], "12": [2, 261], "13": [2, 261], "14": [2, 261], "15": [2, 261], "18": [2, 261], "19": [2, 261], "23": [2, 261], "26": [2, 261], "48": [2, 261], "62": [2, 261], "64": [2, 261], "92": [2, 261], "112": [2, 261], "113": [2, 261], "130": [2, 261], "131": [2, 261] }, { "1": [2, 259], "5": [2, 259], "7": [2, 259], "9": [2, 259], "10": [2, 259], "11": [2, 259], "12": [2, 259], "13": [2, 259], "14": [2, 259], "15": [2, 259], "18": [2, 259], "19": [2, 259], "23": [2, 259], "26": [2, 259], "48": [2, 259], "62": [2, 259], "64": [2, 259], "92": [2, 259], "112": [2, 259], "113": [2, 259], "130": [2, 259], "131": [2, 259] }, { "60": [1, 244] }, { "1": [2, 236], "5": [2, 236], "7": [2, 236], "9": [2, 236], "10": [2, 236], "11": [2, 236], "12": [2, 236], "13": [2, 236], "14": [2, 236], "15": [2, 236], "18": [2, 236], "19": [2, 236], "23": [2, 236], "26": [2, 236], "48": [2, 236], "62": [2, 236], "64": [2, 236], "92": [2, 236], "112": [2, 236], "113": [2, 236], "130": [2, 236], "131": [2, 236] }, { "1": [2, 237], "5": [2, 237], "7": [2, 237], "9": [2, 237], "10": [2, 237], "11": [2, 237], "12": [2, 237], "13": [2, 237], "14": [2, 237], "15": [2, 237], "18": [2, 237], "19": [2, 237], "23": [2, 237], "26": [2, 237], "48": [2, 237], "62": [2, 237], "64": [2, 237], "92": [2, 237], "112": [2, 237], "113": [2, 237], "130": [2, 237], "131": [2, 237] }, { "1": [2, 238], "5": [2, 238], "7": [2, 238], "9": [2, 238], "10": [2, 238], "11": [2, 238], "12": [2, 238], "13": [2, 238], "14": [2, 238], "15": [2, 238], "18": [2, 238], "19": [2, 238], "23": [2, 238], "26": [2, 238], "48": [2, 238], "62": [2, 238], "64": [2, 238], "92": [2, 238], "112": [2, 238], "113": [2, 238], "130": [2, 238], "131": [2, 238] }, { "1": [2, 234], "5": [2, 234], "7": [2, 234], "9": [2, 234], "10": [2, 234], "11": [2, 234], "12": [2, 234], "13": [2, 234], "14": [2, 234], "15": [2, 234], "18": [2, 234], "19": [2, 234], "23": [2, 234], "26": [2, 234], "48": [2, 234], "62": [2, 234], "64": [2, 234], "92": [2, 234], "112": [2, 234], "113": [2, 234], "130": [2, 234], "131": [2, 234] }, { "1": [2, 235], "5": [2, 235], "7": [2, 235], "9": [2, 235], "10": [2, 235], "11": [2, 235], "12": [2, 235], "13": [2, 235], "14": [2, 235], "15": [2, 235], "18": [2, 235], "19": [2, 235], "23": [2, 235], "26": [2, 235], "48": [2, 235], "62": [2, 235], "64": [2, 235], "92": [2, 235], "112": [2, 235], "113": [2, 235], "130": [2, 235], "131": [2, 235] }, { "1": [2, 239], "5": [2, 239], "7": [2, 239], "9": [2, 239], "10": [2, 239], "11": [2, 239], "12": [2, 239], "13": [2, 239], "14": [2, 239], "15": [2, 239], "18": [2, 239], "19": [2, 239], "23": [2, 239], "26": [2, 239], "48": [2, 239], "62": [2, 239], "64": [2, 239], "92": [2, 239], "112": [2, 239], "113": [2, 239], "130": [2, 239], "131": [2, 239] }, { "1": [2, 241], "5": [2, 241], "7": [2, 241], "9": [2, 241], "10": [2, 241], "11": [2, 241], "12": [2, 241], "13": [2, 241], "14": [2, 241], "15": [2, 241], "18": [2, 241], "19": [2, 241], "23": [2, 241], "26": [2, 241], "48": [2, 241], "62": [2, 241], "64": [2, 241], "92": [2, 241], "112": [2, 241], "113": [2, 241], "130": [2, 241], "131": [2, 241] }, { "1": [2, 246], "5": [2, 246], "7": [2, 246], "9": [2, 246], "10": [2, 246], "11": [2, 246], "12": [2, 246], "13": [2, 246], "14": [2, 246], "15": [2, 246], "18": [2, 246], "19": [2, 246], "23": [2, 246], "26": [2, 246], "48": [2, 246], "62": [2, 246], "64": [2, 246], "92": [2, 246], "112": [2, 246], "113": [2, 246], "130": [2, 246], "131": [2, 246] }, { "57": [1, 245] }, { "57": [1, 246] }, { "1": [2, 226], "5": [2, 226], "7": [2, 226], "9": [2, 226], "10": [2, 226], "11": [2, 226], "12": [2, 226], "13": [2, 226], "14": [2, 226], "15": [2, 226], "18": [2, 226], "19": [2, 226], "23": [2, 226], "26": [2, 226], "48": [2, 226], "62": [2, 226], "64": [2, 226], "92": [2, 226], "112": [2, 226], "113": [2, 226], "130": [2, 226], "131": [2, 226] }, { "1": [2, 227], "5": [2, 227], "7": [2, 227], "9": [2, 227], "10": [2, 227], "11": [2, 227], "12": [2, 227], "13": [2, 227], "14": [2, 227], "15": [2, 227], "18": [2, 227], "19": [2, 227], "23": [2, 227], "26": [2, 227], "48": [2, 227], "62": [2, 227], "64": [2, 227], "92": [2, 227], "112": [2, 227], "113": [2, 227], "130": [2, 227], "131": [2, 227] }, { "1": [2, 228], "5": [2, 228], "7": [2, 228], "9": [2, 228], "10": [2, 228], "11": [2, 228], "12": [2, 228], "13": [2, 228], "14": [2, 228], "15": [2, 228], "18": [2, 228], "19": [2, 228], "23": [2, 228], "26": [2, 228], "48": [2, 228], "62": [2, 228], "64": [2, 228], "92": [2, 228], "112": [2, 228], "113": [2, 228], "130": [2, 228], "131": [2, 228] }, { "1": [2, 229], "5": [2, 229], "7": [2, 229], "9": [2, 229], "10": [2, 229], "11": [2, 229], "12": [2, 229], "13": [2, 229], "14": [2, 229], "15": [2, 229], "18": [2, 229], "19": [2, 229], "23": [2, 229], "26": [2, 229], "48": [2, 229], "62": [2, 229], "64": [2, 229], "92": [2, 229], "112": [2, 229], "113": [2, 229], "130": [2, 229], "131": [2, 229] }, { "3": [1, 247] }, { "1": [2, 223], "5": [2, 223], "7": [2, 223], "9": [2, 223], "10": [2, 223], "11": [2, 223], "12": [2, 223], "13": [2, 223], "14": [2, 223], "15": [2, 223], "18": [2, 223], "19": [2, 223], "23": [2, 223], "26": [2, 223], "48": [2, 223], "62": [2, 223], "64": [2, 223], "92": [2, 223], "112": [2, 223], "113": [2, 223], "130": [2, 223], "131": [2, 223] }, { "1": [2, 221], "5": [2, 221], "7": [2, 221], "9": [2, 221], "10": [2, 221], "11": [2, 221], "12": [2, 221], "13": [2, 221], "14": [2, 221], "15": [2, 221], "18": [2, 221], "19": [2, 221], "23": [2, 221], "26": [2, 221], "30": [1, 248], "48": [2, 221], "57": [1, 249], "62": [2, 221], "64": [2, 221], "92": [2, 221], "112": [2, 221], "113": [2, 221], "130": [2, 221], "131": [2, 221] }, { "1": [2, 214], "5": [2, 214], "7": [2, 214], "9": [2, 214], "10": [2, 214], "11": [2, 214], "12": [2, 214], "13": [2, 214], "14": [2, 214], "15": [2, 214], "18": [2, 214], "19": [2, 214], "23": [2, 214], "26": [2, 214], "30": [2, 214], "48": [2, 214], "57": [2, 214], "62": [2, 214], "64": [2, 214], "92": [2, 214], "112": [2, 214], "113": [2, 214], "130": [2, 214], "131": [2, 214] }, { "1": [2, 215], "5": [2, 215], "7": [2, 215], "9": [2, 215], "10": [2, 215], "11": [2, 215], "12": [2, 215], "13": [2, 215], "14": [2, 215], "15": [2, 215], "18": [2, 215], "19": [2, 215], "23": [2, 215], "26": [2, 215], "30": [2, 215], "48": [2, 215], "57": [2, 215], "62": [2, 215], "64": [2, 215], "92": [2, 215], "112": [2, 215], "113": [2, 215], "130": [2, 215], "131": [2, 215] }, { "1": [2, 216], "5": [2, 216], "7": [2, 216], "9": [2, 216], "10": [2, 216], "11": [2, 216], "12": [2, 216], "13": [2, 216], "14": [2, 216], "15": [2, 216], "18": [2, 216], "19": [2, 216], "23": [2, 216], "26": [2, 216], "30": [2, 216], "48": [2, 216], "57": [2, 216], "62": [2, 216], "64": [2, 216], "92": [2, 216], "112": [2, 216], "113": [2, 216], "130": [2, 216], "131": [2, 216] }, { "1": [2, 217], "5": [2, 217], "7": [2, 217], "9": [2, 217], "10": [2, 217], "11": [2, 217], "12": [2, 217], "13": [2, 217], "14": [2, 217], "15": [2, 217], "18": [2, 217], "19": [2, 217], "23": [2, 217], "26": [2, 217], "30": [2, 217], "48": [2, 217], "57": [2, 217], "62": [2, 217], "64": [2, 217], "92": [2, 217], "112": [2, 217], "113": [2, 217], "130": [2, 217], "131": [2, 217] }, { "16": [1, 143], "17": [1, 142], "42": [1, 141], "47": [1, 140] }, { "1": [2, 219], "5": [2, 219], "7": [2, 219], "9": [2, 219], "10": [2, 219], "11": [2, 219], "12": [2, 219], "13": [2, 219], "14": [2, 219], "15": [2, 219], "18": [2, 219], "19": [2, 219], "23": [2, 219], "26": [2, 219], "48": [2, 219], "62": [2, 219], "64": [2, 219], "92": [2, 219], "112": [2, 219], "113": [2, 219], "130": [2, 219], "131": [2, 219] }, { "1": [2, 220], "5": [2, 220], "7": [2, 220], "9": [2, 220], "10": [2, 220], "11": [2, 220], "12": [2, 220], "13": [2, 220], "14": [2, 220], "15": [2, 220], "18": [2, 220], "19": [2, 220], "23": [2, 220], "26": [2, 220], "48": [2, 220], "62": [2, 220], "64": [2, 220], "92": [2, 220], "112": [2, 220], "113": [2, 220], "130": [2, 220], "131": [2, 220] }, { "1": [2, 209], "5": [2, 209], "7": [2, 209], "9": [2, 209], "10": [2, 209], "11": [2, 209], "12": [2, 209], "13": [2, 209], "14": [2, 209], "15": [2, 209], "18": [2, 209], "19": [2, 209], "23": [2, 209], "26": [2, 209], "48": [2, 209], "62": [2, 209], "64": [2, 209], "92": [2, 209], "112": [2, 209], "113": [2, 209], "130": [2, 209], "131": [2, 209] }, { "1": [2, 210], "5": [2, 210], "7": [2, 210], "9": [2, 210], "10": [2, 210], "11": [2, 210], "12": [2, 210], "13": [2, 210], "14": [2, 210], "15": [2, 210], "18": [2, 210], "19": [2, 210], "23": [2, 210], "26": [2, 210], "48": [2, 210], "62": [2, 210], "64": [2, 210], "92": [2, 210], "112": [2, 210], "113": [2, 210], "130": [2, 210], "131": [2, 210] }, { "1": [2, 207], "5": [2, 207], "7": [2, 207], "9": [2, 207], "10": [2, 207], "11": [2, 207], "12": [2, 207], "13": [2, 207], "14": [2, 207], "15": [2, 207], "18": [2, 207], "19": [2, 207], "23": [2, 207], "26": [2, 207], "48": [2, 207], "62": [2, 207], "64": [2, 207], "92": [2, 207], "112": [2, 207], "113": [2, 207], "130": [2, 207], "131": [2, 207] }, { "1": [2, 208], "5": [2, 208], "7": [2, 208], "9": [2, 208], "10": [2, 208], "11": [2, 208], "12": [2, 208], "13": [2, 208], "14": [2, 208], "15": [2, 208], "18": [2, 208], "19": [2, 208], "23": [2, 208], "26": [2, 208], "48": [2, 208], "62": [2, 208], "64": [2, 208], "92": [2, 208], "112": [2, 208], "113": [2, 208], "130": [2, 208], "131": [2, 208] }, { "1": [2, 161], "5": [2, 161], "7": [2, 161], "9": [2, 161], "10": [2, 161], "11": [2, 161], "12": [2, 161], "13": [2, 161], "14": [2, 161], "15": [2, 161], "18": [2, 161], "19": [2, 161], "23": [2, 161], "26": [2, 161], "48": [2, 161], "62": [2, 161], "64": [2, 161], "92": [2, 161], "112": [2, 161], "113": [2, 161], "130": [2, 161], "131": [2, 161] }, { "1": [2, 3] }, { "1": [2, 6] }, { "1": [2, 7] }, { "1": [2, 8] }, { "10": [1, 254], "27": [1, 255], "39": [1, 256], "110": [1, 252] }, { "1": [2, 99], "9": [2, 99], "18": [2, 99], "23": [2, 99], "64": [2, 99], "113": [2, 99], "131": [2, 99] }, { "10": [1, 254], "27": [1, 255], "39": [1, 256], "110": [1, 252] }, { "8": [1, 42], "28": [1, 49], "39": [1, 35], "48": [1, 31], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "103": [1, 55], "104": [1, 54], "108": [1, 30], "123": [1, 56], "128": [1, 162], "129": [1, 50], "130": [1, 48] }, { "8": [1, 42], "28": [1, 49], "39": [1, 35], "48": [1, 31], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "103": [1, 55], "104": [1, 54], "108": [1, 30], "123": [1, 56], "128": [1, 162], "129": [1, 50], "130": [1, 48] }, { "1": [2, 98], "9": [2, 98], "18": [2, 98], "23": [2, 98], "64": [2, 98], "113": [2, 98], "131": [2, 98] }, { "48": [1, 157] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "8": [1, 42], "28": [1, 49], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "103": [1, 55], "104": [1, 54], "123": [1, 56], "129": [1, 50], "130": [1, 48] }, { "3": [2, 105], "4": [2, 105], "8": [2, 105], "14": [2, 105], "16": [2, 105], "17": [2, 105], "28": [2, 105], "29": [2, 105], "32": [2, 105], "33": [2, 105], "34": [2, 105], "36": [2, 105], "38": [2, 105], "39": [2, 105], "40": [2, 105], "41": [2, 105], "42": [2, 105], "43": [2, 105], "44": [2, 105], "46": [2, 105], "47": [2, 105], "48": [2, 105], "49": [2, 105], "50": [2, 105], "51": [2, 105], "52": [2, 105], "54": [2, 105], "56": [2, 105], "58": [2, 105], "59": [2, 105], "61": [2, 105], "68": [2, 105], "78": [2, 105], "81": [2, 105], "85": [2, 105], "91": [2, 105], "99": [2, 105], "100": [2, 105], "101": [2, 105], "102": [2, 105], "103": [2, 105], "104": [2, 105], "107": [2, 105], "110": [2, 105], "119": [2, 105], "123": [2, 105], "125": [2, 105], "126": [2, 105], "128": [2, 105], "129": [2, 105], "130": [2, 105] }, { "3": [2, 106], "4": [2, 106], "8": [2, 106], "14": [2, 106], "16": [2, 106], "17": [2, 106], "28": [2, 106], "29": [2, 106], "32": [2, 106], "33": [2, 106], "34": [2, 106], "36": [2, 106], "38": [2, 106], "39": [2, 106], "40": [2, 106], "41": [2, 106], "42": [2, 106], "43": [2, 106], "44": [2, 106], "46": [2, 106], "47": [2, 106], "48": [2, 106], "49": [2, 106], "50": [2, 106], "51": [2, 106], "52": [2, 106], "54": [2, 106], "56": [2, 106], "58": [2, 106], "59": [2, 106], "61": [2, 106], "68": [2, 106], "78": [2, 106], "81": [2, 106], "85": [2, 106], "91": [2, 106], "99": [2, 106], "100": [2, 106], "101": [2, 106], "102": [2, 106], "103": [2, 106], "104": [2, 106], "107": [2, 106], "110": [2, 106], "119": [2, 106], "123": [2, 106], "125": [2, 106], "126": [2, 106], "128": [2, 106], "129": [2, 106], "130": [2, 106] }, { "3": [2, 107], "4": [2, 107], "8": [2, 107], "14": [2, 107], "16": [2, 107], "17": [2, 107], "28": [2, 107], "29": [2, 107], "32": [2, 107], "33": [2, 107], "34": [2, 107], "36": [2, 107], "38": [2, 107], "39": [2, 107], "40": [2, 107], "41": [2, 107], "42": [2, 107], "43": [2, 107], "44": [2, 107], "46": [2, 107], "47": [2, 107], "48": [2, 107], "49": [2, 107], "50": [2, 107], "51": [2, 107], "52": [2, 107], "54": [2, 107], "56": [2, 107], "58": [2, 107], "59": [2, 107], "61": [2, 107], "68": [2, 107], "78": [2, 107], "81": [2, 107], "85": [2, 107], "91": [2, 107], "99": [2, 107], "100": [2, 107], "101": [2, 107], "102": [2, 107], "103": [2, 107], "104": [2, 107], "107": [2, 107], "110": [2, 107], "119": [2, 107], "123": [2, 107], "125": [2, 107], "126": [2, 107], "128": [2, 107], "129": [2, 107], "130": [2, 107] }, { "3": [2, 108], "4": [2, 108], "8": [2, 108], "14": [2, 108], "16": [2, 108], "17": [2, 108], "28": [2, 108], "29": [2, 108], "32": [2, 108], "33": [2, 108], "34": [2, 108], "36": [2, 108], "38": [2, 108], "39": [2, 108], "40": [2, 108], "41": [2, 108], "42": [2, 108], "43": [2, 108], "44": [2, 108], "46": [2, 108], "47": [2, 108], "48": [2, 108], "49": [2, 108], "50": [2, 108], "51": [2, 108], "52": [2, 108], "54": [2, 108], "56": [2, 108], "58": [2, 108], "59": [2, 108], "61": [2, 108], "68": [2, 108], "78": [2, 108], "81": [2, 108], "85": [2, 108], "91": [2, 108], "99": [2, 108], "100": [2, 108], "101": [2, 108], "102": [2, 108], "103": [2, 108], "104": [2, 108], "107": [2, 108], "110": [2, 108], "119": [2, 108], "123": [2, 108], "125": [2, 108], "126": [2, 108], "128": [2, 108], "129": [2, 108], "130": [2, 108] }, { "3": [2, 109], "4": [2, 109], "8": [2, 109], "14": [2, 109], "16": [2, 109], "17": [2, 109], "28": [2, 109], "29": [2, 109], "32": [2, 109], "33": [2, 109], "34": [2, 109], "36": [2, 109], "38": [2, 109], "39": [2, 109], "40": [2, 109], "41": [2, 109], "42": [2, 109], "43": [2, 109], "44": [2, 109], "46": [2, 109], "47": [2, 109], "48": [2, 109], "49": [2, 109], "50": [2, 109], "51": [2, 109], "52": [2, 109], "54": [2, 109], "56": [2, 109], "58": [2, 109], "59": [2, 109], "61": [2, 109], "68": [2, 109], "78": [2, 109], "81": [2, 109], "85": [2, 109], "91": [2, 109], "99": [2, 109], "100": [2, 109], "101": [2, 109], "102": [2, 109], "103": [2, 109], "104": [2, 109], "107": [2, 109], "110": [2, 109], "119": [2, 109], "123": [2, 109], "125": [2, 109], "126": [2, 109], "128": [2, 109], "129": [2, 109], "130": [2, 109] }, { "3": [2, 110], "4": [2, 110], "8": [2, 110], "14": [2, 110], "16": [2, 110], "17": [2, 110], "28": [2, 110], "29": [2, 110], "32": [2, 110], "33": [2, 110], "34": [2, 110], "36": [2, 110], "38": [2, 110], "39": [2, 110], "40": [2, 110], "41": [2, 110], "42": [2, 110], "43": [2, 110], "44": [2, 110], "46": [2, 110], "47": [2, 110], "48": [2, 110], "49": [2, 110], "50": [2, 110], "51": [2, 110], "52": [2, 110], "54": [2, 110], "56": [2, 110], "58": [2, 110], "59": [2, 110], "61": [2, 110], "68": [2, 110], "78": [2, 110], "81": [2, 110], "85": [2, 110], "91": [2, 110], "99": [2, 110], "100": [2, 110], "101": [2, 110], "102": [2, 110], "103": [2, 110], "104": [2, 110], "107": [2, 110], "110": [2, 110], "119": [2, 110], "123": [2, 110], "125": [2, 110], "126": [2, 110], "128": [2, 110], "129": [2, 110], "130": [2, 110] }, { "3": [2, 111], "4": [2, 111], "8": [2, 111], "14": [2, 111], "16": [2, 111], "17": [2, 111], "28": [2, 111], "29": [2, 111], "32": [2, 111], "33": [2, 111], "34": [2, 111], "36": [2, 111], "38": [2, 111], "39": [2, 111], "40": [2, 111], "41": [2, 111], "42": [2, 111], "43": [2, 111], "44": [2, 111], "46": [2, 111], "47": [2, 111], "48": [2, 111], "49": [2, 111], "50": [2, 111], "51": [2, 111], "52": [2, 111], "54": [2, 111], "56": [2, 111], "58": [2, 111], "59": [2, 111], "61": [2, 111], "68": [2, 111], "78": [2, 111], "81": [2, 111], "85": [2, 111], "91": [2, 111], "99": [2, 111], "100": [2, 111], "101": [2, 111], "102": [2, 111], "103": [2, 111], "104": [2, 111], "107": [2, 111], "110": [2, 111], "119": [2, 111], "123": [2, 111], "125": [2, 111], "126": [2, 111], "128": [2, 111], "129": [2, 111], "130": [2, 111] }, { "3": [2, 112], "4": [2, 112], "8": [2, 112], "14": [2, 112], "16": [2, 112], "17": [2, 112], "28": [2, 112], "29": [2, 112], "32": [2, 112], "33": [2, 112], "34": [2, 112], "36": [2, 112], "38": [2, 112], "39": [2, 112], "40": [2, 112], "41": [2, 112], "42": [2, 112], "43": [2, 112], "44": [2, 112], "46": [2, 112], "47": [2, 112], "48": [2, 112], "49": [2, 112], "50": [2, 112], "51": [2, 112], "52": [2, 112], "54": [2, 112], "56": [2, 112], "58": [2, 112], "59": [2, 112], "61": [2, 112], "68": [2, 112], "78": [2, 112], "81": [2, 112], "85": [2, 112], "91": [2, 112], "99": [2, 112], "100": [2, 112], "101": [2, 112], "102": [2, 112], "103": [2, 112], "104": [2, 112], "107": [2, 112], "110": [2, 112], "119": [2, 112], "123": [2, 112], "125": [2, 112], "126": [2, 112], "128": [2, 112], "129": [2, 112], "130": [2, 112] }, { "3": [2, 113], "4": [2, 113], "8": [2, 113], "14": [2, 113], "16": [2, 113], "17": [2, 113], "28": [2, 113], "29": [2, 113], "32": [2, 113], "33": [2, 113], "34": [2, 113], "36": [2, 113], "38": [2, 113], "39": [2, 113], "40": [2, 113], "41": [2, 113], "42": [2, 113], "43": [2, 113], "44": [2, 113], "46": [2, 113], "47": [2, 113], "48": [2, 113], "49": [2, 113], "50": [2, 113], "51": [2, 113], "52": [2, 113], "54": [2, 113], "56": [2, 113], "58": [2, 113], "59": [2, 113], "61": [2, 113], "68": [2, 113], "78": [2, 113], "81": [2, 113], "85": [2, 113], "91": [2, 113], "99": [2, 113], "100": [2, 113], "101": [2, 113], "102": [2, 113], "103": [2, 113], "104": [2, 113], "107": [2, 113], "110": [2, 113], "119": [2, 113], "123": [2, 113], "125": [2, 113], "126": [2, 113], "128": [2, 113], "129": [2, 113], "130": [2, 113] }, { "3": [2, 114], "4": [2, 114], "8": [2, 114], "14": [2, 114], "16": [2, 114], "17": [2, 114], "28": [2, 114], "29": [2, 114], "32": [2, 114], "33": [2, 114], "34": [2, 114], "36": [2, 114], "38": [2, 114], "39": [2, 114], "40": [2, 114], "41": [2, 114], "42": [2, 114], "43": [2, 114], "44": [2, 114], "46": [2, 114], "47": [2, 114], "48": [2, 114], "49": [2, 114], "50": [2, 114], "51": [2, 114], "52": [2, 114], "54": [2, 114], "56": [2, 114], "58": [2, 114], "59": [2, 114], "61": [2, 114], "68": [2, 114], "78": [2, 114], "81": [2, 114], "85": [2, 114], "91": [2, 114], "99": [2, 114], "100": [2, 114], "101": [2, 114], "102": [2, 114], "103": [2, 114], "104": [2, 114], "107": [2, 114], "110": [2, 114], "119": [2, 114], "123": [2, 114], "125": [2, 114], "126": [2, 114], "128": [2, 114], "129": [2, 114], "130": [2, 114] }, { "3": [2, 115], "4": [2, 115], "8": [2, 115], "14": [2, 115], "16": [2, 115], "17": [2, 115], "28": [2, 115], "29": [2, 115], "32": [2, 115], "33": [2, 115], "34": [2, 115], "36": [2, 115], "38": [2, 115], "39": [2, 115], "40": [2, 115], "41": [2, 115], "42": [2, 115], "43": [2, 115], "44": [2, 115], "46": [2, 115], "47": [2, 115], "48": [2, 115], "49": [2, 115], "50": [2, 115], "51": [2, 115], "52": [2, 115], "54": [2, 115], "56": [2, 115], "58": [2, 115], "59": [2, 115], "61": [2, 115], "68": [2, 115], "78": [2, 115], "81": [2, 115], "85": [2, 115], "91": [2, 115], "99": [2, 115], "100": [2, 115], "101": [2, 115], "102": [2, 115], "103": [2, 115], "104": [2, 115], "107": [2, 115], "110": [2, 115], "119": [2, 115], "123": [2, 115], "125": [2, 115], "126": [2, 115], "128": [2, 115], "129": [2, 115], "130": [2, 115] }, { "3": [2, 116], "4": [2, 116], "8": [2, 116], "14": [2, 116], "16": [2, 116], "17": [2, 116], "28": [2, 116], "29": [2, 116], "32": [2, 116], "33": [2, 116], "34": [2, 116], "36": [2, 116], "38": [2, 116], "39": [2, 116], "40": [2, 116], "41": [2, 116], "42": [2, 116], "43": [2, 116], "44": [2, 116], "46": [2, 116], "47": [2, 116], "48": [2, 116], "49": [2, 116], "50": [2, 116], "51": [2, 116], "52": [2, 116], "54": [2, 116], "56": [2, 116], "58": [2, 116], "59": [2, 116], "61": [2, 116], "68": [2, 116], "78": [2, 116], "81": [2, 116], "85": [2, 116], "91": [2, 116], "99": [2, 116], "100": [2, 116], "101": [2, 116], "102": [2, 116], "103": [2, 116], "104": [2, 116], "107": [2, 116], "110": [2, 116], "119": [2, 116], "123": [2, 116], "125": [2, 116], "126": [2, 116], "128": [2, 116], "129": [2, 116], "130": [2, 116] }, { "3": [2, 117], "4": [2, 117], "8": [2, 117], "14": [2, 117], "16": [2, 117], "17": [2, 117], "28": [2, 117], "29": [2, 117], "32": [2, 117], "33": [2, 117], "34": [2, 117], "36": [2, 117], "38": [2, 117], "39": [2, 117], "40": [2, 117], "41": [2, 117], "42": [2, 117], "43": [2, 117], "44": [2, 117], "46": [2, 117], "47": [2, 117], "48": [2, 117], "49": [2, 117], "50": [2, 117], "51": [2, 117], "52": [2, 117], "54": [2, 117], "56": [2, 117], "58": [2, 117], "59": [2, 117], "61": [2, 117], "68": [2, 117], "78": [2, 117], "81": [2, 117], "85": [2, 117], "91": [2, 117], "99": [2, 117], "100": [2, 117], "101": [2, 117], "102": [2, 117], "103": [2, 117], "104": [2, 117], "107": [2, 117], "110": [2, 117], "119": [2, 117], "123": [2, 117], "125": [2, 117], "126": [2, 117], "128": [2, 117], "129": [2, 117], "130": [2, 117] }, { "3": [2, 118], "4": [2, 118], "8": [2, 118], "14": [2, 118], "16": [2, 118], "17": [2, 118], "28": [2, 118], "29": [2, 118], "32": [2, 118], "33": [2, 118], "34": [2, 118], "36": [2, 118], "38": [2, 118], "39": [2, 118], "40": [2, 118], "41": [2, 118], "42": [2, 118], "43": [2, 118], "44": [2, 118], "46": [2, 118], "47": [2, 118], "48": [2, 118], "49": [2, 118], "50": [2, 118], "51": [2, 118], "52": [2, 118], "54": [2, 118], "56": [2, 118], "58": [2, 118], "59": [2, 118], "61": [2, 118], "68": [2, 118], "78": [2, 118], "81": [2, 118], "85": [2, 118], "91": [2, 118], "99": [2, 118], "100": [2, 118], "101": [2, 118], "102": [2, 118], "103": [2, 118], "104": [2, 118], "107": [2, 118], "110": [2, 118], "119": [2, 118], "123": [2, 118], "125": [2, 118], "126": [2, 118], "128": [2, 118], "129": [2, 118], "130": [2, 118] }, { "3": [2, 119], "4": [2, 119], "8": [2, 119], "14": [2, 119], "16": [2, 119], "17": [2, 119], "28": [2, 119], "29": [2, 119], "32": [2, 119], "33": [2, 119], "34": [2, 119], "36": [2, 119], "38": [2, 119], "39": [2, 119], "40": [2, 119], "41": [2, 119], "42": [2, 119], "43": [2, 119], "44": [2, 119], "46": [2, 119], "47": [2, 119], "48": [2, 119], "49": [2, 119], "50": [2, 119], "51": [2, 119], "52": [2, 119], "54": [2, 119], "56": [2, 119], "58": [2, 119], "59": [2, 119], "61": [2, 119], "68": [2, 119], "78": [2, 119], "81": [2, 119], "85": [2, 119], "91": [2, 119], "99": [2, 119], "100": [2, 119], "101": [2, 119], "102": [2, 119], "103": [2, 119], "104": [2, 119], "107": [2, 119], "110": [2, 119], "119": [2, 119], "123": [2, 119], "125": [2, 119], "126": [2, 119], "128": [2, 119], "129": [2, 119], "130": [2, 119] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "8": [1, 42], "28": [1, 49], "39": [1, 35], "48": [1, 31], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "103": [1, 55], "104": [1, 54], "108": [1, 30], "123": [1, 56], "128": [1, 162], "129": [1, 50], "130": [1, 48] }, { "1": [2, 86], "5": [2, 86], "9": [2, 86], "19": [2, 86], "23": [2, 86], "48": [2, 86], "112": [2, 86], "130": [2, 86] }, { "21": [1, 264] }, { "1": [2, 83], "5": [2, 83], "9": [2, 83], "19": [2, 83], "23": [2, 83], "26": [2, 83], "48": [2, 83], "112": [2, 83], "130": [2, 83] }, { "21": [1, 265] }, { "8": [1, 42], "28": [1, 49], "48": [1, 73], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "103": [1, 55], "104": [1, 54], "123": [1, 56], "129": [1, 50], "130": [1, 48] }, { "8": [1, 42], "28": [1, 49], "48": [1, 73], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "103": [1, 55], "104": [1, 54], "123": [1, 56], "129": [1, 50], "130": [1, 48] }, { "8": [1, 42], "28": [1, 49], "48": [1, 73], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "103": [1, 55], "104": [1, 54], "123": [1, 56], "129": [1, 50], "130": [1, 48] }, { "8": [1, 42], "28": [1, 49], "48": [1, 73], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "103": [1, 55], "104": [1, 54], "123": [1, 56], "129": [1, 50], "130": [1, 48] }, { "8": [1, 42], "28": [1, 49], "48": [1, 73], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "103": [1, 55], "104": [1, 54], "123": [1, 56], "129": [1, 50], "130": [1, 48] }, { "8": [1, 42], "28": [1, 49], "48": [1, 73], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "103": [1, 55], "104": [1, 54], "123": [1, 56], "129": [1, 50], "130": [1, 48] }, { "130": [1, 272] }, { "9": [1, 273] }, { "1": [2, 165], "5": [2, 165], "7": [2, 165], "9": [2, 165], "10": [2, 165], "11": [2, 165], "12": [2, 165], "13": [2, 165], "14": [2, 165], "15": [2, 165], "18": [2, 165], "19": [2, 165], "20": [2, 165], "22": [2, 165], "23": [2, 165], "24": [2, 165], "25": [2, 165], "26": [2, 165], "48": [2, 165], "62": [2, 165], "64": [2, 165], "76": [2, 165], "77": [2, 165], "86": [2, 165], "92": [2, 165], "94": [2, 165], "95": [2, 165], "111": [2, 165], "112": [2, 165], "113": [2, 165], "115": [2, 165], "120": [2, 165], "122": [2, 165], "130": [2, 165], "131": [2, 165], "132": [2, 165], "133": [2, 165], "134": [2, 165] }, { "13": [1, 275], "131": [1, 274] }, { "21": [1, 276] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "1": [2, 163], "5": [2, 163], "7": [2, 163], "9": [2, 163], "10": [2, 163], "11": [2, 163], "12": [2, 163], "13": [2, 163], "14": [2, 163], "15": [2, 163], "18": [2, 163], "19": [2, 163], "20": [2, 163], "22": [2, 163], "23": [2, 163], "24": [2, 163], "25": [2, 163], "26": [2, 163], "48": [2, 163], "62": [2, 163], "64": [2, 163], "76": [2, 163], "77": [2, 163], "86": [2, 163], "92": [2, 163], "94": [2, 163], "95": [2, 163], "111": [2, 163], "112": [2, 163], "113": [2, 163], "115": [2, 163], "120": [2, 163], "122": [2, 163], "130": [2, 163], "131": [2, 163], "132": [2, 163], "133": [2, 163], "134": [2, 163] }, { "13": [1, 282], "62": [1, 281] }, { "13": [2, 205], "62": [2, 205] }, { "1": [2, 11], "8": [1, 19], "61": [1, 16], "67": [1, 13], "74": [1, 17], "82": [1, 15], "105": [1, 14], "110": [1, 11], "127": [1, 12] }, { "13": [1, 226], "19": [1, 286] }, { "1": [2, 44], "5": [2, 44], "19": [2, 44], "112": [2, 44] }, { "21": [1, 287] }, { "18": [1, 288] }, { "1": [2, 41], "5": [2, 41], "19": [2, 41], "112": [2, 41] }, { "1": [2, 80], "5": [2, 80], "19": [2, 80], "112": [2, 80] }, { "1": [2, 81], "5": [2, 81], "19": [2, 81], "48": [1, 183], "112": [2, 81] }, { "23": [1, 289] }, { "1": [2, 43], "5": [2, 43], "19": [2, 43], "112": [2, 43] }, { "1": [2, 81], "5": [2, 81], "9": [2, 51], "19": [2, 81], "23": [2, 51], "48": [1, 183], "112": [2, 81] }, { "8": [1, 214], "39": [1, 35], "61": [1, 217], "63": [1, 215], "74": [1, 218], "117": [1, 216] }, { "68": [1, 57], "78": [1, 292], "103": [1, 55], "104": [1, 54], "123": [1, 56] }, { "48": [1, 293] }, { "48": [1, 71] }, { "8": [1, 42], "28": [1, 49], "48": [1, 73], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "103": [1, 55], "104": [1, 54], "123": [1, 56], "129": [1, 50], "130": [1, 48] }, { "9": [2, 62], "23": [2, 62], "112": [1, 296] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "9": [1, 299] }, { "9": [2, 51], "23": [2, 51], "48": [1, 183] }, { "9": [1, 300] }, { "111": [1, 301] }, { "48": [1, 71] }, { "8": [1, 303] }, { "9": [2, 79], "23": [2, 79], "112": [2, 79] }, { "23": [1, 304], "97": [1, 305] }, { "8": [1, 306] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 71], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "130": [1, 314] }, { "8": [1, 315] }, { "8": [1, 316] }, { "8": [1, 317] }, { "8": [1, 318] }, { "3": [1, 319], "60": [1, 320] }, { "3": [2, 212], "60": [2, 212] }, { "1": [2, 256], "5": [2, 256], "7": [2, 256], "9": [2, 256], "10": [2, 256], "11": [2, 256], "12": [2, 256], "13": [2, 256], "14": [2, 256], "15": [2, 256], "18": [2, 256], "19": [2, 256], "23": [2, 256], "26": [2, 256], "48": [2, 256], "62": [2, 256], "64": [2, 256], "92": [2, 256], "112": [2, 256], "113": [2, 256], "130": [2, 256], "131": [2, 256] }, { "1": [2, 257], "5": [2, 257], "7": [2, 257], "9": [2, 257], "10": [2, 257], "11": [2, 257], "12": [2, 257], "13": [2, 257], "14": [2, 257], "15": [2, 257], "18": [2, 257], "19": [2, 257], "23": [2, 257], "26": [2, 257], "48": [2, 257], "62": [2, 257], "64": [2, 257], "92": [2, 257], "112": [2, 257], "113": [2, 257], "130": [2, 257], "131": [2, 257] }, { "60": [1, 244] }, { "1": [2, 225], "5": [2, 225], "7": [2, 225], "9": [2, 225], "10": [2, 225], "11": [2, 225], "12": [2, 225], "13": [2, 225], "14": [2, 225], "15": [2, 225], "18": [2, 225], "19": [2, 225], "23": [2, 225], "26": [2, 225], "48": [2, 225], "62": [2, 225], "64": [2, 225], "92": [2, 225], "112": [2, 225], "113": [2, 225], "130": [2, 225], "131": [2, 225] }, { "1": [2, 218], "5": [2, 218], "7": [2, 218], "9": [2, 218], "10": [2, 218], "11": [2, 218], "12": [2, 218], "13": [2, 218], "14": [2, 218], "15": [2, 218], "18": [2, 218], "19": [2, 218], "23": [2, 218], "26": [2, 218], "48": [2, 218], "62": [2, 218], "64": [2, 218], "92": [2, 218], "112": [2, 218], "113": [2, 218], "130": [2, 218], "131": [2, 218] }, { "1": [2, 222], "5": [2, 222], "7": [2, 222], "9": [2, 222], "10": [2, 222], "11": [2, 222], "12": [2, 222], "13": [2, 222], "14": [2, 222], "15": [2, 222], "18": [2, 222], "19": [2, 222], "23": [2, 222], "26": [2, 222], "48": [2, 222], "62": [2, 222], "64": [2, 222], "92": [2, 222], "112": [2, 222], "113": [2, 222], "130": [2, 222], "131": [2, 222] }, { "1": [2, 32] }, { "23": [1, 322] }, { "23": [1, 323] }, { "1": [2, 37], "23": [2, 37] }, { "1": [2, 38], "23": [2, 38] }, { "1": [2, 39], "23": [2, 39], "92": [1, 324] }, { "1": [2, 33] }, { "1": [2, 94], "9": [2, 94], "18": [2, 94], "23": [2, 94], "64": [2, 94], "113": [1, 160], "131": [2, 94] }, { "1": [2, 96], "9": [2, 96], "18": [2, 96], "23": [2, 96], "64": [2, 96], "113": [2, 96], "131": [2, 96] }, { "1": [2, 100], "9": [2, 100], "18": [2, 100], "23": [2, 100], "64": [2, 100], "113": [2, 100], "131": [2, 100] }, { "7": [2, 130], "10": [2, 130], "11": [2, 130], "12": [2, 130], "14": [2, 130], "15": [2, 130], "20": [2, 130], "22": [2, 130], "24": [2, 130], "25": [2, 130], "76": [2, 130], "77": [2, 130], "86": [2, 130], "94": [2, 130], "95": [2, 130], "111": [2, 130], "115": [2, 130], "120": [2, 130], "122": [2, 130], "132": [2, 130], "133": [2, 130], "134": [2, 130] }, { "1": [2, 101], "9": [2, 101], "18": [2, 101], "23": [2, 101], "64": [2, 101], "113": [2, 101], "131": [2, 101] }, { "64": [1, 159], "131": [1, 325] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "7": [1, 190], "10": [1, 188], "12": [2, 122], "14": [2, 122], "15": [1, 189], "20": [2, 122], "22": [2, 122], "24": [2, 122], "25": [2, 122], "76": [2, 122], "77": [2, 122], "86": [2, 122], "94": [2, 122], "95": [2, 122], "111": [2, 122], "115": [2, 122], "120": [2, 122], "122": [2, 122], "132": [2, 122], "133": [2, 122], "134": [2, 122] }, { "7": [1, 190], "10": [1, 188], "12": [2, 123], "14": [2, 123], "15": [1, 189], "20": [2, 123], "22": [2, 123], "24": [2, 123], "25": [2, 123], "76": [2, 123], "77": [2, 123], "86": [2, 123], "94": [2, 123], "95": [2, 123], "111": [2, 123], "115": [2, 123], "120": [2, 123], "122": [2, 123], "132": [2, 123], "133": [2, 123], "134": [2, 123] }, { "7": [2, 125], "10": [2, 125], "11": [1, 191], "12": [2, 125], "14": [2, 125], "15": [2, 125], "20": [2, 125], "22": [2, 125], "24": [2, 125], "25": [2, 125], "76": [2, 125], "77": [2, 125], "86": [2, 125], "94": [2, 125], "95": [2, 125], "111": [2, 125], "115": [2, 125], "120": [2, 125], "122": [2, 125], "132": [2, 125], "133": [2, 125], "134": [2, 125] }, { "7": [2, 126], "10": [2, 126], "11": [1, 191], "12": [2, 126], "14": [2, 126], "15": [2, 126], "20": [2, 126], "22": [2, 126], "24": [2, 126], "25": [2, 126], "76": [2, 126], "77": [2, 126], "86": [2, 126], "94": [2, 126], "95": [2, 126], "111": [2, 126], "115": [2, 126], "120": [2, 126], "122": [2, 126], "132": [2, 126], "133": [2, 126], "134": [2, 126] }, { "7": [2, 127], "10": [2, 127], "11": [1, 191], "12": [2, 127], "14": [2, 127], "15": [2, 127], "20": [2, 127], "22": [2, 127], "24": [2, 127], "25": [2, 127], "76": [2, 127], "77": [2, 127], "86": [2, 127], "94": [2, 127], "95": [2, 127], "111": [2, 127], "115": [2, 127], "120": [2, 127], "122": [2, 127], "132": [2, 127], "133": [2, 127], "134": [2, 127] }, { "7": [2, 129], "10": [2, 129], "11": [2, 129], "12": [2, 129], "14": [2, 129], "15": [2, 129], "20": [2, 129], "22": [2, 129], "24": [2, 129], "25": [2, 129], "76": [2, 129], "77": [2, 129], "86": [2, 129], "94": [2, 129], "95": [2, 129], "111": [2, 129], "115": [2, 129], "120": [2, 129], "122": [2, 129], "132": [2, 129], "133": [2, 129], "134": [2, 129] }, { "8": [1, 42], "28": [1, 49], "39": [1, 35], "48": [1, 31], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "103": [1, 55], "104": [1, 54], "108": [1, 30], "123": [1, 56], "128": [1, 162], "129": [1, 50], "130": [1, 48] }, { "1": [2, 134], "5": [2, 134], "7": [2, 134], "9": [2, 134], "10": [2, 134], "11": [2, 134], "12": [2, 134], "13": [2, 134], "14": [2, 134], "15": [2, 134], "18": [2, 134], "19": [2, 134], "20": [2, 134], "22": [2, 134], "23": [2, 134], "24": [2, 134], "25": [2, 134], "26": [2, 134], "48": [2, 134], "62": [2, 134], "64": [2, 134], "76": [2, 134], "77": [2, 134], "86": [2, 134], "92": [2, 134], "94": [2, 134], "95": [2, 134], "111": [2, 134], "112": [2, 134], "113": [2, 134], "115": [2, 134], "120": [2, 134], "122": [2, 134], "130": [2, 134], "131": [2, 134], "132": [2, 134], "133": [2, 134], "134": [2, 134] }, { "1": [2, 166], "5": [2, 166], "7": [2, 166], "9": [2, 166], "10": [2, 166], "11": [2, 166], "12": [2, 166], "13": [2, 166], "14": [2, 166], "15": [2, 166], "18": [2, 166], "19": [2, 166], "20": [2, 166], "22": [2, 166], "23": [2, 166], "24": [2, 166], "25": [2, 166], "26": [2, 166], "48": [2, 166], "62": [2, 166], "64": [2, 166], "76": [2, 166], "77": [2, 166], "86": [2, 166], "92": [2, 166], "94": [2, 166], "95": [2, 166], "111": [2, 166], "112": [2, 166], "113": [2, 166], "115": [2, 166], "120": [2, 166], "122": [2, 166], "130": [2, 166], "131": [2, 166], "132": [2, 166], "133": [2, 166], "134": [2, 166] }, { "48": [1, 329] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "9": [1, 331], "13": [1, 332] }, { "9": [2, 103], "13": [2, 103] }, { "9": [1, 333] }, { "9": [1, 334] }, { "1": [2, 164], "5": [2, 164], "7": [2, 164], "9": [2, 164], "10": [2, 164], "11": [2, 164], "12": [2, 164], "13": [2, 164], "14": [2, 164], "15": [2, 164], "18": [2, 164], "19": [2, 164], "20": [2, 164], "22": [2, 164], "23": [2, 164], "24": [2, 164], "25": [2, 164], "26": [2, 164], "48": [2, 164], "62": [2, 164], "64": [2, 164], "76": [2, 164], "77": [2, 164], "86": [2, 164], "92": [2, 164], "94": [2, 164], "95": [2, 164], "111": [2, 164], "112": [2, 164], "113": [2, 164], "115": [2, 164], "120": [2, 164], "122": [2, 164], "130": [2, 164], "131": [2, 164], "132": [2, 164], "133": [2, 164], "134": [2, 164] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "1": [2, 9], "8": [1, 19], "61": [1, 16], "67": [1, 13], "74": [1, 17], "82": [1, 15], "105": [1, 14], "110": [1, 11], "127": [1, 12] }, { "1": [2, 13], "8": [2, 13], "61": [2, 13], "67": [2, 13], "74": [2, 13], "82": [2, 13], "105": [2, 13], "110": [2, 13], "127": [2, 13] }, { "5": [1, 339], "19": [1, 337], "112": [1, 61] }, { "1": [2, 12], "8": [1, 19], "61": [1, 16], "67": [1, 13], "74": [1, 17], "82": [1, 15], "105": [1, 14], "110": [1, 11], "127": [1, 12] }, { "48": [1, 71], "88": [1, 344] }, { "8": [1, 19], "61": [1, 16], "67": [1, 13], "74": [1, 17], "82": [1, 15], "105": [1, 14], "110": [1, 11], "127": [1, 12] }, { "39": [1, 35], "109": [1, 209] }, { "9": [1, 347] }, { "48": [1, 348] }, { "48": [1, 350], "111": [1, 349] }, { "66": [1, 351], "80": [1, 352] }, { "13": [1, 226], "62": [1, 353] }, { "111": [1, 354] }, { "48": [1, 206] }, { "13": [1, 356] }, { "9": [2, 67], "13": [1, 357], "23": [2, 67] }, { "9": [2, 69], "23": [2, 69], "112": [1, 358] }, { "112": [1, 359] }, { "8": [1, 360] }, { "13": [2, 92], "19": [2, 92], "62": [2, 92] }, { "8": [1, 19], "61": [1, 16], "67": [1, 13], "74": [1, 17], "82": [1, 15], "105": [1, 14], "127": [1, 12] }, { "8": [1, 362] }, { "8": [1, 363] }, { "8": [1, 214], "39": [1, 35], "61": [1, 217], "63": [1, 215], "74": [1, 218], "117": [1, 216] }, { "1": [2, 145], "5": [2, 145], "7": [1, 235], "9": [2, 145], "10": [1, 233], "12": [2, 145], "13": [2, 145], "14": [2, 145], "15": [1, 234], "18": [2, 145], "19": [2, 145], "23": [2, 145], "26": [2, 145], "48": [2, 145], "62": [2, 145], "64": [2, 145], "112": [2, 145], "113": [2, 145], "130": [2, 145], "131": [2, 145] }, { "1": [2, 146], "5": [2, 146], "7": [1, 235], "9": [2, 146], "10": [1, 233], "12": [2, 146], "13": [2, 146], "14": [2, 146], "15": [1, 234], "18": [2, 146], "19": [2, 146], "23": [2, 146], "26": [2, 146], "48": [2, 146], "62": [2, 146], "64": [2, 146], "112": [2, 146], "113": [2, 146], "130": [2, 146], "131": [2, 146] }, { "1": [2, 148], "5": [2, 148], "7": [2, 148], "9": [2, 148], "10": [2, 148], "11": [1, 236], "12": [2, 148], "13": [2, 148], "14": [2, 148], "15": [2, 148], "18": [2, 148], "19": [2, 148], "23": [2, 148], "26": [2, 148], "48": [2, 148], "62": [2, 148], "64": [2, 148], "112": [2, 148], "113": [2, 148], "130": [2, 148], "131": [2, 148] }, { "1": [2, 149], "5": [2, 149], "7": [2, 149], "9": [2, 149], "10": [2, 149], "11": [1, 236], "12": [2, 149], "13": [2, 149], "14": [2, 149], "15": [2, 149], "18": [2, 149], "19": [2, 149], "23": [2, 149], "26": [2, 149], "48": [2, 149], "62": [2, 149], "64": [2, 149], "112": [2, 149], "113": [2, 149], "130": [2, 149], "131": [2, 149] }, { "1": [2, 150], "5": [2, 150], "7": [2, 150], "9": [2, 150], "10": [2, 150], "11": [1, 236], "12": [2, 150], "13": [2, 150], "14": [2, 150], "15": [2, 150], "18": [2, 150], "19": [2, 150], "23": [2, 150], "26": [2, 150], "48": [2, 150], "62": [2, 150], "64": [2, 150], "112": [2, 150], "113": [2, 150], "130": [2, 150], "131": [2, 150] }, { "1": [2, 152], "5": [2, 152], "7": [2, 152], "9": [2, 152], "10": [2, 152], "11": [2, 152], "12": [2, 152], "13": [2, 152], "14": [2, 152], "15": [2, 152], "18": [2, 152], "19": [2, 152], "23": [2, 152], "26": [2, 152], "48": [2, 152], "62": [2, 152], "64": [2, 152], "112": [2, 152], "113": [2, 152], "130": [2, 152], "131": [2, 152] }, { "1": [2, 153], "5": [2, 153], "7": [2, 153], "9": [2, 153], "10": [2, 153], "11": [2, 153], "12": [2, 153], "13": [2, 153], "14": [2, 153], "15": [2, 153], "18": [2, 153], "19": [2, 153], "23": [2, 153], "26": [2, 153], "48": [2, 153], "62": [2, 153], "64": [2, 153], "112": [2, 153], "113": [2, 153], "130": [2, 153], "131": [2, 153] }, { "8": [1, 42], "28": [1, 49], "39": [1, 35], "48": [1, 31], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "103": [1, 55], "104": [1, 54], "108": [1, 30], "123": [1, 56], "128": [1, 162], "129": [1, 50], "130": [1, 48] }, { "130": [1, 368] }, { "3": [1, 371], "9": [1, 369], "13": [1, 372], "14": [1, 144], "16": [1, 143], "17": [1, 142], "38": [1, 373], "42": [1, 141], "47": [1, 140] }, { "3": [1, 377], "4": [1, 147], "14": [1, 144], "16": [1, 143], "17": [1, 142], "42": [1, 141], "47": [1, 140], "52": [1, 148] }, { "16": [1, 143], "17": [1, 142], "42": [1, 141], "47": [1, 140] }, { "1": [2, 211], "5": [2, 211], "7": [2, 211], "9": [2, 211], "10": [2, 211], "11": [2, 211], "12": [2, 211], "13": [2, 211], "14": [2, 211], "15": [2, 211], "18": [2, 211], "19": [2, 211], "23": [2, 211], "26": [2, 211], "37": [1, 379], "48": [2, 211], "62": [2, 211], "64": [2, 211], "92": [2, 211], "112": [2, 211], "113": [2, 211], "130": [2, 211], "131": [2, 211] }, { "3": [2, 213], "60": [2, 213] }, { "3": [1, 380], "60": [1, 320] }, { "10": [1, 254], "27": [1, 255], "39": [1, 256] }, { "10": [1, 254], "27": [1, 255], "39": [1, 256], "109": [1, 382] }, { "8": [1, 42], "28": [1, 49], "39": [1, 35], "48": [1, 31], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "103": [1, 55], "104": [1, 54], "108": [1, 30], "123": [1, 56], "128": [1, 162], "129": [1, 50], "130": [1, 48] }, { "1": [2, 102], "9": [2, 102], "18": [2, 102], "23": [2, 102], "64": [2, 102], "113": [2, 102], "131": [2, 102] }, { "1": [2, 89], "5": [2, 89], "9": [2, 89], "19": [2, 89], "23": [2, 89], "48": [2, 89], "112": [2, 89], "130": [2, 89] }, { "1": [2, 84], "5": [2, 84], "9": [2, 84], "19": [2, 84], "23": [2, 84], "26": [2, 84], "48": [2, 84], "112": [2, 84], "130": [2, 84] }, { "64": [1, 159], "131": [1, 385] }, { "21": [1, 386] }, { "13": [2, 167], "131": [2, 167] }, { "1": [2, 157], "5": [2, 157], "7": [2, 157], "9": [2, 157], "10": [2, 157], "11": [2, 157], "12": [2, 157], "13": [2, 157], "14": [2, 157], "15": [2, 157], "18": [2, 157], "19": [2, 157], "20": [2, 157], "22": [2, 157], "23": [2, 157], "24": [2, 157], "25": [2, 157], "26": [2, 157], "48": [2, 157], "62": [2, 157], "64": [2, 157], "76": [2, 157], "77": [2, 157], "86": [2, 157], "92": [2, 157], "94": [2, 157], "95": [2, 157], "111": [2, 157], "112": [2, 157], "113": [2, 157], "115": [2, 157], "120": [2, 157], "122": [2, 157], "130": [2, 157], "131": [2, 157], "132": [2, 157], "133": [2, 157], "134": [2, 157] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "1": [2, 158], "5": [2, 158], "7": [2, 158], "9": [2, 158], "10": [2, 158], "11": [2, 158], "12": [2, 158], "13": [2, 158], "14": [2, 158], "15": [2, 158], "18": [2, 158], "19": [2, 158], "20": [2, 158], "22": [2, 158], "23": [2, 158], "24": [2, 158], "25": [2, 158], "26": [2, 158], "48": [2, 158], "62": [2, 158], "64": [2, 158], "76": [2, 158], "77": [2, 158], "86": [2, 158], "92": [2, 158], "94": [2, 158], "95": [2, 158], "111": [2, 158], "112": [2, 158], "113": [2, 158], "115": [2, 158], "120": [2, 158], "122": [2, 158], "130": [2, 158], "131": [2, 158], "132": [2, 158], "133": [2, 158], "134": [2, 158] }, { "1": [2, 159], "5": [2, 159], "7": [2, 159], "9": [2, 159], "10": [2, 159], "11": [2, 159], "12": [2, 159], "13": [2, 159], "14": [2, 159], "15": [2, 159], "18": [2, 159], "19": [2, 159], "20": [2, 159], "22": [2, 159], "23": [2, 159], "24": [2, 159], "25": [2, 159], "26": [2, 159], "48": [2, 159], "62": [2, 159], "64": [2, 159], "76": [2, 159], "77": [2, 159], "86": [2, 159], "92": [2, 159], "94": [2, 159], "95": [2, 159], "111": [2, 159], "112": [2, 159], "113": [2, 159], "115": [2, 159], "120": [2, 159], "122": [2, 159], "130": [2, 159], "131": [2, 159], "132": [2, 159], "133": [2, 159], "134": [2, 159] }, { "13": [2, 206], "62": [2, 206] }, { "1": [2, 14], "8": [2, 14], "61": [2, 14], "67": [2, 14], "74": [2, 14], "82": [2, 14], "105": [2, 14], "110": [2, 14], "127": [2, 14] }, { "1": [2, 15], "8": [2, 15], "61": [2, 15], "67": [2, 15], "74": [2, 15], "82": [2, 15], "105": [2, 15], "110": [2, 15], "127": [2, 15] }, { "19": [1, 388] }, { "75": [1, 389], "78": [1, 391], "87": [1, 393], "106": [1, 392], "116": [1, 390] }, { "5": [1, 395], "19": [2, 20] }, { "5": [2, 18], "19": [2, 18] }, { "1": [2, 10], "8": [1, 19], "61": [1, 16], "67": [1, 13], "74": [1, 17], "82": [1, 15], "105": [1, 14], "110": [1, 11], "127": [1, 12] }, { "1": [2, 87], "5": [2, 87], "9": [2, 87], "19": [2, 87], "23": [2, 87], "112": [2, 87] }, { "1": [2, 88], "5": [2, 88], "9": [2, 88], "19": [2, 88], "23": [2, 88], "112": [2, 88] }, { "1": [2, 31], "112": [1, 61] }, { "1": [2, 42], "5": [2, 42], "19": [2, 42], "112": [2, 42] }, { "61": [1, 397], "92": [1, 396], "97": [1, 398] }, { "111": [1, 399] }, { "8": [1, 400] }, { "111": [1, 401] }, { "111": [1, 402] }, { "111": [1, 403] }, { "111": [1, 404] }, { "8": [1, 405] }, { "9": [2, 64], "23": [2, 64], "112": [2, 64] }, { "96": [1, 406] }, { "90": [1, 407] }, { "107": [1, 408] }, { "8": [1, 42], "28": [1, 49], "39": [1, 35], "48": [1, 31], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "103": [1, 55], "104": [1, 54], "108": [1, 30], "123": [1, 56], "128": [1, 410], "129": [1, 50], "130": [1, 48] }, { "8": [1, 19], "61": [1, 16], "67": [1, 13], "74": [1, 17], "82": [1, 15], "105": [1, 14], "127": [1, 12] }, { "9": [1, 412] }, { "8": [1, 214], "39": [1, 35], "61": [1, 217], "63": [1, 215], "74": [1, 218], "117": [1, 216] }, { "8": [1, 214], "39": [1, 35], "61": [1, 217], "63": [1, 215], "74": [1, 218], "117": [1, 216] }, { "9": [1, 415] }, { "64": [1, 159], "131": [1, 416] }, { "9": [1, 417], "13": [1, 418] }, { "9": [2, 170], "13": [2, 170] }, { "70": [1, 421], "71": [1, 423], "79": [1, 427], "83": [1, 422], "84": [1, 424], "93": [1, 425], "96": [1, 426], "121": [1, 428] }, { "1": [2, 240], "5": [2, 240], "7": [2, 240], "9": [2, 240], "10": [2, 240], "11": [2, 240], "12": [2, 240], "13": [2, 240], "14": [2, 240], "15": [2, 240], "18": [2, 240], "19": [2, 240], "23": [2, 240], "26": [2, 240], "48": [2, 240], "62": [2, 240], "64": [2, 240], "92": [2, 240], "112": [2, 240], "113": [2, 240], "130": [2, 240], "131": [2, 240] }, { "9": [1, 430], "13": [1, 429] }, { "60": [1, 244] }, { "13": [1, 433], "14": [1, 144], "16": [1, 143], "17": [1, 142], "42": [1, 141], "47": [1, 140] }, { "9": [1, 434], "13": [1, 435] }, { "1": [2, 221], "5": [2, 221], "7": [2, 221], "9": [2, 221], "10": [2, 221], "11": [2, 221], "12": [2, 221], "13": [2, 221], "14": [2, 221], "15": [2, 221], "18": [2, 221], "19": [2, 221], "23": [2, 221], "26": [2, 221], "48": [2, 221], "62": [2, 221], "64": [2, 221], "92": [2, 221], "112": [2, 221], "113": [2, 221], "130": [2, 221], "131": [2, 221] }, { "13": [1, 436] }, { "9": [1, 437] }, { "60": [1, 244] }, { "13": [1, 439] }, { "1": [2, 260], "5": [2, 260], "7": [2, 260], "9": [2, 260], "10": [2, 260], "11": [2, 260], "12": [2, 260], "13": [2, 260], "14": [2, 260], "15": [2, 260], "18": [2, 260], "19": [2, 260], "23": [2, 260], "26": [2, 260], "48": [2, 260], "62": [2, 260], "64": [2, 260], "92": [2, 260], "112": [2, 260], "113": [2, 260], "130": [2, 260], "131": [2, 260] }, { "1": [2, 230], "5": [2, 230], "7": [2, 230], "9": [2, 230], "10": [2, 230], "11": [2, 230], "12": [2, 230], "13": [2, 230], "14": [2, 230], "15": [2, 230], "18": [2, 230], "19": [2, 230], "23": [2, 230], "26": [2, 230], "48": [2, 230], "62": [2, 230], "64": [2, 230], "92": [2, 230], "112": [2, 230], "113": [2, 230], "130": [2, 230], "131": [2, 230] }, { "1": [2, 34] }, { "1": [2, 35] }, { "1": [2, 36] }, { "1": [2, 40], "23": [2, 40], "64": [1, 159] }, { "7": [2, 133], "10": [2, 133], "11": [2, 133], "12": [2, 133], "14": [2, 133], "15": [2, 133], "20": [2, 133], "22": [2, 133], "24": [2, 133], "25": [2, 133], "76": [2, 133], "77": [2, 133], "86": [2, 133], "94": [2, 133], "95": [2, 133], "111": [2, 133], "115": [2, 133], "120": [2, 133], "122": [2, 133], "132": [2, 133], "133": [2, 133], "134": [2, 133] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "9": [2, 104], "13": [2, 104] }, { "1": [2, 16], "8": [2, 16], "61": [2, 16], "67": [2, 16], "74": [2, 16], "82": [2, 16], "105": [2, 16], "110": [2, 16], "127": [2, 16] }, { "21": [1, 441] }, { "21": [1, 442] }, { "21": [1, 443] }, { "21": [1, 444] }, { "21": [1, 445] }, { "5": [2, 19], "19": [2, 19] }, { "78": [1, 391], "87": [1, 393], "106": [1, 392], "116": [1, 390] }, { "8": [1, 42], "28": [1, 49], "39": [1, 35], "48": [1, 31], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "103": [1, 55], "104": [1, 54], "108": [1, 30], "123": [1, 56], "128": [1, 162], "129": [1, 50], "130": [1, 48] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "8": [1, 449] }, { "8": [1, 450] }, { "8": [1, 214], "39": [1, 35], "61": [1, 217], "63": [1, 215], "74": [1, 218], "117": [1, 216] }, { "8": [1, 452] }, { "8": [1, 453] }, { "8": [1, 454] }, { "8": [1, 455] }, { "8": [1, 214], "39": [1, 35], "61": [1, 217], "63": [1, 215], "74": [1, 218], "117": [1, 216] }, { "21": [1, 457] }, { "21": [1, 458] }, { "48": [1, 71], "61": [1, 460] }, { "9": [2, 72], "23": [2, 72], "64": [1, 159] }, { "9": [2, 73], "23": [2, 73], "48": [1, 157] }, { "9": [1, 461] }, { "9": [2, 75], "23": [2, 75] }, { "9": [1, 462] }, { "9": [1, 463] }, { "1": [2, 45] }, { "1": [2, 156], "5": [2, 156], "7": [2, 156], "9": [2, 156], "10": [2, 156], "11": [2, 156], "12": [2, 156], "13": [2, 156], "14": [2, 156], "15": [2, 156], "18": [2, 156], "19": [2, 156], "23": [2, 156], "26": [2, 156], "48": [2, 156], "62": [2, 156], "64": [2, 156], "112": [2, 156], "113": [2, 156], "130": [2, 156], "131": [2, 156] }, { "1": [2, 169], "5": [2, 169], "7": [2, 169], "9": [2, 169], "10": [2, 169], "11": [2, 169], "12": [2, 169], "13": [2, 169], "14": [2, 169], "15": [2, 169], "18": [2, 169], "19": [2, 169], "23": [2, 169], "26": [2, 169], "48": [2, 169], "62": [2, 169], "64": [2, 169], "92": [2, 169], "112": [2, 169], "113": [2, 169], "130": [2, 169], "131": [2, 169] }, { "130": [1, 368] }, { "13": [1, 466], "131": [1, 465] }, { "13": [2, 173], "131": [2, 173] }, { "21": [1, 467] }, { "21": [1, 468] }, { "21": [1, 469] }, { "21": [1, 470] }, { "21": [1, 471] }, { "21": [1, 472] }, { "21": [1, 473] }, { "21": [1, 474] }, { "13": [1, 476], "14": [1, 144], "16": [1, 143], "17": [1, 142], "42": [1, 141], "47": [1, 140] }, { "1": [2, 244], "5": [2, 244], "7": [2, 244], "9": [2, 244], "10": [2, 244], "11": [2, 244], "12": [2, 244], "13": [2, 244], "14": [2, 244], "15": [2, 244], "18": [2, 244], "19": [2, 244], "23": [2, 244], "26": [2, 244], "48": [2, 244], "62": [2, 244], "64": [2, 244], "92": [2, 244], "112": [2, 244], "113": [2, 244], "130": [2, 244], "131": [2, 244] }, { "3": [1, 477], "60": [1, 320] }, { "13": [1, 478] }, { "14": [1, 144], "16": [1, 143], "17": [1, 142], "42": [1, 141], "47": [1, 140] }, { "1": [2, 254], "5": [2, 254], "7": [2, 254], "9": [2, 254], "10": [2, 254], "11": [2, 254], "12": [2, 254], "13": [2, 254], "14": [2, 254], "15": [2, 254], "18": [2, 254], "19": [2, 254], "23": [2, 254], "26": [2, 254], "48": [2, 254], "62": [2, 254], "64": [2, 254], "92": [2, 254], "112": [2, 254], "113": [2, 254], "130": [2, 254], "131": [2, 254] }, { "43": [1, 127], "56": [1, 126] }, { "14": [1, 144], "16": [1, 143], "17": [1, 142], "42": [1, 141], "47": [1, 140] }, { "1": [2, 233], "5": [2, 233], "7": [2, 233], "9": [2, 233], "10": [2, 233], "11": [2, 233], "12": [2, 233], "13": [2, 233], "14": [2, 233], "15": [2, 233], "18": [2, 233], "19": [2, 233], "23": [2, 233], "26": [2, 233], "48": [2, 233], "62": [2, 233], "64": [2, 233], "92": [2, 233], "112": [2, 233], "113": [2, 233], "130": [2, 233], "131": [2, 233] }, { "3": [1, 482], "60": [1, 320] }, { "57": [1, 483] }, { "13": [2, 168], "131": [2, 168] }, { "38": [1, 484] }, { "61": [1, 486] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "128": [1, 488] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "9": [2, 52], "23": [2, 52], "64": [1, 159] }, { "13": [1, 282], "62": [1, 490] }, { "13": [2, 205], "18": [1, 491], "62": [2, 205] }, { "8": [1, 214], "39": [1, 35], "61": [1, 217], "63": [1, 215], "74": [1, 218], "117": [1, 216] }, { "8": [1, 214], "39": [1, 35], "61": [1, 217], "63": [1, 215], "74": [1, 218], "117": [1, 216] }, { "9": [1, 494] }, { "8": [1, 214], "39": [1, 35], "61": [1, 217], "63": [1, 215], "74": [1, 218], "117": [1, 216] }, { "8": [1, 214], "39": [1, 35], "61": [1, 217], "63": [1, 215], "74": [1, 218], "117": [1, 216] }, { "8": [1, 214], "39": [1, 35], "61": [1, 217], "63": [1, 215], "74": [1, 218], "117": [1, 216] }, { "8": [1, 214], "39": [1, 35], "61": [1, 217], "63": [1, 215], "74": [1, 218], "117": [1, 216] }, { "9": [1, 499] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "9": [2, 70], "23": [2, 70] }, { "48": [1, 71] }, { "9": [2, 74], "23": [2, 74] }, { "9": [2, 77], "23": [2, 77], "112": [2, 77] }, { "9": [2, 78], "23": [2, 78], "112": [2, 78] }, { "9": [2, 171], "13": [2, 171] }, { "9": [2, 172], "13": [2, 172] }, { "70": [1, 421], "71": [1, 423], "79": [1, 427], "83": [1, 422], "84": [1, 424], "93": [1, 425], "96": [1, 426], "121": [1, 428] }, { "32": [1, 130], "85": [1, 132], "107": [1, 505], "119": [1, 131] }, { "32": [1, 130], "85": [1, 132], "107": [1, 505], "119": [1, 131] }, { "43": [1, 127], "56": [1, 126] }, { "43": [1, 127], "56": [1, 126] }, { "16": [1, 143], "17": [1, 142], "42": [1, 141], "47": [1, 140] }, { "16": [1, 143], "17": [1, 142], "34": [1, 146], "42": [1, 141], "46": [1, 145], "47": [1, 140] }, { "38": [1, 512] }, { "91": [1, 514], "128": [1, 513] }, { "13": [1, 515] }, { "13": [1, 516] }, { "9": [1, 517] }, { "13": [1, 518], "14": [1, 144], "16": [1, 143], "17": [1, 142], "42": [1, 141], "47": [1, 140] }, { "13": [1, 520] }, { "9": [1, 521] }, { "9": [1, 523], "13": [1, 522] }, { "1": [2, 211], "5": [2, 211], "7": [2, 211], "9": [2, 211], "10": [2, 211], "11": [2, 211], "12": [2, 211], "13": [2, 211], "14": [2, 211], "15": [2, 211], "18": [2, 211], "19": [2, 211], "23": [2, 211], "26": [2, 211], "48": [2, 211], "62": [2, 211], "64": [2, 211], "92": [2, 211], "112": [2, 211], "113": [2, 211], "130": [2, 211], "131": [2, 211] }, { "9": [1, 524] }, { "62": [1, 525] }, { "62": [1, 526] }, { "62": [1, 527], "130": [1, 48] }, { "62": [1, 530] }, { "62": [1, 531] }, { "62": [1, 532] }, { "9": [2, 58], "23": [2, 58] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "9": [1, 534] }, { "9": [1, 535] }, { "9": [2, 54], "23": [2, 54] }, { "9": [1, 536] }, { "9": [1, 537] }, { "9": [1, 538] }, { "9": [1, 539] }, { "9": [2, 61], "23": [2, 61] }, { "9": [2, 65], "13": [1, 540], "23": [2, 65] }, { "9": [2, 68], "23": [2, 68] }, { "13": [1, 226], "62": [1, 541] }, { "13": [2, 174], "131": [2, 174] }, { "13": [2, 175], "131": [2, 175] }, { "35": [1, 542] }, { "13": [2, 176], "131": [2, 176] }, { "13": [2, 177], "131": [2, 177] }, { "13": [2, 178], "131": [2, 178] }, { "13": [2, 179], "131": [2, 179] }, { "13": [2, 180], "131": [2, 180] }, { "57": [1, 249] }, { "13": [2, 181], "131": [2, 181] }, { "13": [2, 182], "131": [2, 182] }, { "13": [2, 183], "131": [2, 183] }, { "13": [1, 544], "14": [1, 144], "16": [1, 143], "17": [1, 142], "42": [1, 141], "47": [1, 140] }, { "9": [1, 545] }, { "1": [2, 245], "5": [2, 245], "7": [2, 245], "9": [2, 245], "10": [2, 245], "11": [2, 245], "12": [2, 245], "13": [2, 245], "14": [2, 245], "15": [2, 245], "18": [2, 245], "19": [2, 245], "23": [2, 245], "26": [2, 245], "48": [2, 245], "62": [2, 245], "64": [2, 245], "92": [2, 245], "112": [2, 245], "113": [2, 245], "130": [2, 245], "131": [2, 245] }, { "9": [1, 546] }, { "13": [1, 547] }, { "9": [1, 548], "43": [1, 127], "56": [1, 126] }, { "1": [2, 255], "5": [2, 255], "7": [2, 255], "9": [2, 255], "10": [2, 255], "11": [2, 255], "12": [2, 255], "13": [2, 255], "14": [2, 255], "15": [2, 255], "18": [2, 255], "19": [2, 255], "23": [2, 255], "26": [2, 255], "48": [2, 255], "62": [2, 255], "64": [2, 255], "92": [2, 255], "112": [2, 255], "113": [2, 255], "130": [2, 255], "131": [2, 255] }, { "3": [1, 377], "4": [1, 147], "52": [1, 148] }, { "1": [2, 232], "5": [2, 232], "7": [2, 232], "9": [2, 232], "10": [2, 232], "11": [2, 232], "12": [2, 232], "13": [2, 232], "14": [2, 232], "15": [2, 232], "18": [2, 232], "19": [2, 232], "23": [2, 232], "26": [2, 232], "48": [2, 232], "62": [2, 232], "64": [2, 232], "92": [2, 232], "112": [2, 232], "113": [2, 232], "130": [2, 232], "131": [2, 232] }, { "1": [2, 224], "5": [2, 224], "7": [2, 224], "9": [2, 224], "10": [2, 224], "11": [2, 224], "12": [2, 224], "13": [2, 224], "14": [2, 224], "15": [2, 224], "18": [2, 224], "19": [2, 224], "23": [2, 224], "26": [2, 224], "48": [2, 224], "62": [2, 224], "64": [2, 224], "92": [2, 224], "112": [2, 224], "113": [2, 224], "130": [2, 224], "131": [2, 224] }, { "19": [1, 551] }, { "5": [2, 21], "19": [2, 21] }, { "62": [2, 25] }, { "13": [1, 553], "62": [1, 552] }, { "13": [2, 27], "62": [2, 27] }, { "5": [2, 22], "19": [2, 22] }, { "5": [2, 23], "19": [2, 23] }, { "5": [2, 24], "19": [2, 24] }, { "62": [1, 554] }, { "9": [2, 63], "23": [2, 63], "112": [2, 63] }, { "9": [2, 53], "23": [2, 53] }, { "9": [2, 55], "23": [2, 55] }, { "9": [2, 56], "23": [2, 56] }, { "9": [2, 57], "23": [2, 57] }, { "9": [2, 60], "23": [2, 60] }, { "93": [1, 555] }, { "9": [2, 71], "23": [2, 71] }, { "8": [1, 556] }, { "9": [1, 557], "13": [1, 558] }, { "9": [1, 559] }, { "1": [2, 247], "5": [2, 247], "7": [2, 247], "9": [2, 247], "10": [2, 247], "11": [2, 247], "12": [2, 247], "13": [2, 247], "14": [2, 247], "15": [2, 247], "18": [2, 247], "19": [2, 247], "23": [2, 247], "26": [2, 247], "48": [2, 247], "62": [2, 247], "64": [2, 247], "92": [2, 247], "112": [2, 247], "113": [2, 247], "130": [2, 247], "131": [2, 247] }, { "1": [2, 248], "5": [2, 248], "7": [2, 248], "9": [2, 248], "10": [2, 248], "11": [2, 248], "12": [2, 248], "13": [2, 248], "14": [2, 248], "15": [2, 248], "18": [2, 248], "19": [2, 248], "23": [2, 248], "26": [2, 248], "48": [2, 248], "62": [2, 248], "64": [2, 248], "92": [2, 248], "112": [2, 248], "113": [2, 248], "130": [2, 248], "131": [2, 248] }, { "9": [1, 560], "43": [1, 127], "56": [1, 126] }, { "1": [2, 249], "5": [2, 249], "7": [2, 249], "9": [2, 249], "10": [2, 249], "11": [2, 249], "12": [2, 249], "13": [2, 249], "14": [2, 249], "15": [2, 249], "18": [2, 249], "19": [2, 249], "23": [2, 249], "26": [2, 249], "48": [2, 249], "62": [2, 249], "64": [2, 249], "92": [2, 249], "112": [2, 249], "113": [2, 249], "130": [2, 249], "131": [2, 249] }, { "9": [1, 562] }, { "9": [1, 563] }, { "1": [2, 17], "8": [2, 17], "61": [2, 17], "67": [2, 17], "74": [2, 17], "82": [2, 17], "105": [2, 17], "110": [2, 17], "127": [2, 17] }, { "62": [2, 26] }, { "130": [1, 48] }, { "9": [2, 59], "23": [2, 59] }, { "21": [1, 565] }, { "3": [1, 371], "13": [1, 372], "14": [1, 144], "16": [1, 143], "17": [1, 142], "38": [1, 373], "42": [1, 141], "47": [1, 140] }, { "1": [2, 242], "5": [2, 242], "7": [2, 242], "9": [2, 242], "10": [2, 242], "11": [2, 242], "12": [2, 242], "13": [2, 242], "14": [2, 242], "15": [2, 242], "18": [2, 242], "19": [2, 242], "23": [2, 242], "26": [2, 242], "48": [2, 242], "62": [2, 242], "64": [2, 242], "92": [2, 242], "112": [2, 242], "113": [2, 242], "130": [2, 242], "131": [2, 242] }, { "14": [1, 144], "16": [1, 143], "17": [1, 142], "42": [1, 141], "47": [1, 140] }, { "1": [2, 250], "5": [2, 250], "7": [2, 250], "9": [2, 250], "10": [2, 250], "11": [2, 250], "12": [2, 250], "13": [2, 250], "14": [2, 250], "15": [2, 250], "18": [2, 250], "19": [2, 250], "23": [2, 250], "26": [2, 250], "48": [2, 250], "62": [2, 250], "64": [2, 250], "92": [2, 250], "112": [2, 250], "113": [2, 250], "130": [2, 250], "131": [2, 250] }, { "1": [2, 251], "5": [2, 251], "7": [2, 251], "9": [2, 251], "10": [2, 251], "11": [2, 251], "12": [2, 251], "13": [2, 251], "14": [2, 251], "15": [2, 251], "18": [2, 251], "19": [2, 251], "23": [2, 251], "26": [2, 251], "48": [2, 251], "62": [2, 251], "64": [2, 251], "92": [2, 251], "112": [2, 251], "113": [2, 251], "130": [2, 251], "131": [2, 251] }, { "9": [1, 567] }, { "1": [2, 252], "5": [2, 252], "7": [2, 252], "9": [2, 252], "10": [2, 252], "11": [2, 252], "12": [2, 252], "13": [2, 252], "14": [2, 252], "15": [2, 252], "18": [2, 252], "19": [2, 252], "23": [2, 252], "26": [2, 252], "48": [2, 252], "62": [2, 252], "64": [2, 252], "92": [2, 252], "112": [2, 252], "113": [2, 252], "130": [2, 252], "131": [2, 252] }, { "1": [2, 231], "5": [2, 231], "7": [2, 231], "9": [2, 231], "10": [2, 231], "11": [2, 231], "12": [2, 231], "13": [2, 231], "14": [2, 231], "15": [2, 231], "18": [2, 231], "19": [2, 231], "23": [2, 231], "26": [2, 231], "48": [2, 231], "62": [2, 231], "64": [2, 231], "92": [2, 231], "112": [2, 231], "113": [2, 231], "130": [2, 231], "131": [2, 231] }, { "13": [2, 28], "62": [2, 28] }, { "3": [1, 122], "4": [1, 147], "8": [1, 42], "14": [1, 144], "16": [1, 143], "17": [1, 142], "28": [1, 49], "29": [1, 138], "32": [1, 130], "33": [1, 114], "34": [1, 146], "36": [1, 117], "38": [1, 111], "39": [1, 113], "40": [1, 121], "41": [1, 119], "42": [1, 141], "43": [1, 127], "44": [1, 136], "46": [1, 145], "47": [1, 140], "48": [1, 83], "49": [1, 116], "50": [1, 118], "51": [1, 112], "52": [1, 148], "54": [1, 109], "56": [1, 126], "58": [1, 115], "59": [1, 120], "61": [1, 59], "68": [1, 57], "78": [1, 53], "81": [1, 58], "85": [1, 132], "91": [1, 150], "99": [1, 137], "100": [1, 133], "101": [1, 134], "102": [1, 135], "103": [1, 55], "104": [1, 54], "107": [1, 110], "110": [1, 128], "119": [1, 131], "123": [1, 56], "125": [1, 125], "126": [1, 124], "128": [1, 149], "129": [1, 50], "130": [1, 48] }, { "13": [1, 569] }, { "1": [2, 253], "5": [2, 253], "7": [2, 253], "9": [2, 253], "10": [2, 253], "11": [2, 253], "12": [2, 253], "13": [2, 253], "14": [2, 253], "15": [2, 253], "18": [2, 253], "19": [2, 253], "23": [2, 253], "26": [2, 253], "48": [2, 253], "62": [2, 253], "64": [2, 253], "92": [2, 253], "112": [2, 253], "113": [2, 253], "130": [2, 253], "131": [2, 253] }, { "9": [2, 66], "23": [2, 66] }, { "14": [1, 144], "16": [1, 143], "17": [1, 142], "42": [1, 141], "47": [1, 140] }, { "13": [1, 571] }, { "14": [1, 144], "16": [1, 143], "17": [1, 142], "42": [1, 141], "47": [1, 140] }, { "9": [1, 573] }, { "1": [2, 243], "5": [2, 243], "7": [2, 243], "9": [2, 243], "10": [2, 243], "11": [2, 243], "12": [2, 243], "13": [2, 243], "14": [2, 243], "15": [2, 243], "18": [2, 243], "19": [2, 243], "23": [2, 243], "26": [2, 243], "48": [2, 243], "62": [2, 243], "64": [2, 243], "92": [2, 243], "112": [2, 243], "113": [2, 243], "130": [2, 243], "131": [2, 243] }];
const SEMANTIC_ACTION = [
    (($, $0) => { return $0; }),
    (($, $0, $1, s) => new Ast.ControlCommand($.location, new Ast.ControlIntent.Special($.location, s.value))),
    (($, $0, $1, c) => new Ast.ControlCommand($.location, new Ast.ControlIntent.Answer($.location, c))),
    (($, $0, $1, n) => new Ast.ControlCommand($.location, new Ast.ControlIntent.Choice($.location, n))),
    (($, $0, p) => p),
    (($, $0) => { return $0; }),
    (($, $0) => 0),
    (($, $0) => 1),
    (($, n) => n.value),
    (($, $0, dialogueAct, $1, history) => {
        return new Ast.DialogueState($.location, dialogueAct.value.kind, dialogueAct.value.channel, null, history);
    }),
    (($, $0, dialogueAct, pnames, $1, history) => {
        return new Ast.DialogueState($.location, dialogueAct.value.kind, dialogueAct.value.channel, pnames.map((p) => p.name), history);
    }),
    (($, $0, dialogueAct, $1) => {
        return new Ast.DialogueState($.location, dialogueAct.value.kind, dialogueAct.value.channel, null, []);
    }),
    (($, $0, dialogueAct, pnames, $1) => {
        return new Ast.DialogueState($.location, dialogueAct.value.kind, dialogueAct.value.channel, pnames.map((p) => p.name), []);
    }),
    (($, item) => [item]),
    (($, list, item) => {
        list.push(item);
        return list;
    }),
    (($, rule, $0) => new Ast.DialogueHistoryItem($.location, rule, null, 'accepted')),
    (($, rule, results, $0) => new Ast.DialogueHistoryItem($.location, rule, results, 'confirmed')),
    (($, rule, $0, $1, $2, venum, $3, $4) => new Ast.DialogueHistoryItem($.location, rule, null, venum.value)),
    (($, ann) => [ann]),
    (($, list, ann) => list.concat([ann])),
    (($, list) => {
        let results, _error, more, count;
        for (const annot of list) {
            switch (annot.key) {
                case 'results':
                    if (results !== undefined)
                        return $.error(`Duplicate history annotation #[results]`);
                    results = annot.value;
                    break;
                case 'error':
                    if (_error !== undefined)
                        return $.error(`Duplicate history annotation #[results]`);
                    _error = annot.value;
                    break;
                case 'more':
                    if (more !== undefined)
                        return $.error(`Duplicate history annotation #[more]`);
                    more = annot.value;
                    break;
                case 'count':
                    if (count !== undefined)
                        return $.error(`Duplicate history annotation #[count]`);
                    count = annot.value;
                    break;
            }
        }
        if (results === undefined)
            return $.error(`Missing history annotation #[results]`);
        if (count === undefined)
            count = new Ast.Value.Number(results.length);
        return new Ast.DialogueHistoryResultList($.location, results, count, more, _error);
    }),
    (($, $0, $1, $2, results, $3) => ({ key: 'results', value: results })),
    (($, $0, $1, $2, count, $3) => ({ key: 'count', value: count })),
    (($, $0, $1, $2, $3, $4) => ({ key: 'more', value: true })),
    (($, $0, $1, $2, error, $3) => ({ key: 'error', value: error })),
    (($, $0, $1) => []),
    (($, $0, list, $1) => list),
    (($, value) => [new Ast.DialogueHistoryResultItem($.location, value.value)]),
    (($, list, $0, value) => {
        list.push(new Ast.DialogueHistoryResultItem(null, value.value));
        return list;
    }),
    (($, assignment) => new Ast.Program($.location, [], [], [assignment])),
    (($, rule) => new Ast.Program($.location, [], [], [rule])),
    (($, $0, $1, user, $2, rule) => new Ast.Program(null, [], [], [rule], { impl: { executor: user } })),
    (($, $0, $1, policy) => policy),
    (($, user, $0, policy) => {
        // replace all uses of "source" with "$source"
        user.visit(new class extends Ast.NodeVisitor {
            visitComputeBooleanExpression(atom) {
                if (atom.lhs instanceof Ast.VarRefValue && atom.lhs.name === 'source')
                    atom.lhs = new Ast.EventValue('source');
                return true;
            }
        });
        policy.principal = user;
        return policy;
    }),
    (($, $0, $1, action) => {
        return new Ast.PermissionRule($.location, Ast.BooleanExpression.True, Ast.PermissionFunction.Builtin, action);
    }),
    (($, query, $0, $1) => {
        return new Ast.PermissionRule($.location, Ast.BooleanExpression.True, query, Ast.PermissionFunction.Builtin);
    }),
    (($, query, $0, action) => {
        return new Ast.PermissionRule($.location, Ast.BooleanExpression.True, query, action);
    }),
    (($, $0) => Ast.PermissionFunction.Star),
    (($, klass) => new Ast.PermissionFunction.ClassStar($.location, klass.value)),
    (($, fn) => new Ast.PermissionFunction.Specified($.location, fn.value.kind, fn.value.channel, Ast.BooleanExpression.True, null)),
    (($, fn, $0, filter) => new Ast.PermissionFunction.Specified($.location, fn.value.kind, fn.value.channel, filter, null)),
    (($, stream, $0, action) => new Ast.Rule($.location, stream, [action]).toExpression()),
    (($, $0, $1, table, $2, action) => new Ast.Command($.location, table, [action]).toExpression()),
    (($, $0, $1, action) => new Ast.Command($.location, null, [action]).toExpression()),
    (($, rule, $0, pp) => {
        rule = rule.clone();
        const action = rule.last;
        assert_1.default(action instanceof Ast.InvocationExpression);
        action.invocation.in_params.push(pp);
        return rule;
    }),
    (($, $0, name, $1, $2, table, $3) => new Ast.Assignment($.location, name.value, table.toExpression([]))),
    (($, $0) => 'min'),
    (($, $0) => 'max'),
    (($, $0) => 'sum'),
    (($, $0) => 'avg'),
    (($, $0) => 'distance'),
    (($, call) => new Ast.Table.Invocation($.location, call, null)),
    (($, $0, table, $1, $2, filter) => {
        return new Ast.Table.Filter($.location, table, filter, table.schema);
    }),
    (($, $0, op, pname, $1, $2, table, $3) => new Ast.Table.Aggregation($.location, table, pname.value, op, null, null)),
    (($, $0, $1, $2, $3, table, $4) => new Ast.Table.Aggregation($.location, table, '*', 'count', null, null)),
    (($, $0, $1, pname, $2, $3, table, $4) => new Ast.Table.Aggregation($.location, table, pname.value, 'count', null, null)),
    (($, $0, pname, $1, $2, $3, table, $4) => new Ast.Table.Sort($.location, table, pname.value, 'asc', null)),
    (($, $0, pname, $1, $2, $3, table, $4) => new Ast.Table.Sort($.location, table, pname.value, 'desc', null)),
    (($, $0, table, $1, $2, indices, $3) => new Ast.Table.Index($.location, table, indices, null)),
    (($, $0, table, $1, $2, base, $3, delta, $4) => new Ast.Table.Slice($.location, table, base, delta, null)),
    (($, $0, pnames, $1, $2, $3, table, $4) => new Ast.Table.Projection($.location, table, pnames.map((p) => p.name), null)),
    (($, $0, expr, $1, $2, table, $3) => new Ast.Table.Compute($.location, table, expr, null, null)),
    (($, $0) => { return $0; }),
    (($, $0, t1, $1, $2, $3, t2, $4) => new Ast.Table.Join($.location, t1, t2, [], null)),
    (($, join, $0, pp) => {
        join = join.clone();
        join.in_params.push(pp);
        return join;
    }),
    (($, $0, $1, $2, base, $3, $4, $5, interval) => new Ast.Stream.Timer($.location, base, interval, null, null)),
    (($, $0, $1, $2, base, $3, $4, $5, interval, $6, $7, $8, frequency) => new Ast.Stream.Timer($.location, base, interval, frequency, null)),
    (($, $0, $1, $2, time) => new Ast.Stream.AtTimer($.location, time instanceof Ast.ArrayValue ? time.value : [time], null, null)),
    (($, $0, $1, $2, time, $3, $4, $5, expiration_date) => new Ast.Stream.AtTimer($.location, time instanceof Ast.ArrayValue ? time.value : [time], expiration_date, null)),
    (($, $0, $1, table, $2) => new Ast.Stream.Monitor($.location, table, null, null)),
    (($, $0, $1, table, $2, $3, $4, pname) => {
        return new Ast.Stream.Monitor($.location, table, [pname.name], null);
    }),
    (($, $0, $1, table, $2, $3, $4, $5, pnames, $6) => {
        return new Ast.Stream.Monitor($.location, table, pnames.map((p) => p.name), null);
    }),
    (($, $0, $1, stream, $2, $3, filter) => {
        return new Ast.Stream.EdgeFilter($.location, stream, filter, null);
    }),
    (($, $0, $1, stream, $2, $3, $4) => new Ast.Stream.EdgeFilter($.location, stream, Ast.BooleanExpression.True, null)),
    (($, $0, pnames, $1, $2, $3, stream, $4) => new Ast.Stream.Projection($.location, stream, pnames.map((p) => p.name), null)),
    (($, $0, expr, $1, $2, stream, $3) => new Ast.Stream.Compute($.location, stream, expr, null, null)),
    (($, $0) => { return $0; }),
    (($, $0, stream, $1, $2, $3, table, $4) => new Ast.Stream.Join($.location, stream, table, [], null)),
    (($, $0, stream, $1, $2, $3, table, $4) => new Ast.Stream.Join($.location, stream, table, [], null)),
    (($, join, $0, pp) => {
        join = join.clone();
        join.in_params.push(pp);
        return join;
    }),
    (($, $0) => new Ast.Action.Notify($.location, 'notify', null)),
    (($, call) => new Ast.Action.Invocation($.location, call, null)),
    (($, fn) => [new Ast.DeviceSelector($.location, fn.value.kind, null, null), fn.value.channel]),
    (($, selector, attr) => {
        const [sel,] = selector;
        if (attr.name === 'id') {
            sel.id = String(attr.value.toJS());
            const value = attr.value;
            if (value instanceof Ast.EntityValue && value.display)
                sel.attributes.push(new Ast.InputParam(null, 'name', new Ast.Value.String(value.display)));
        }
        else if (attr.name === 'all') {
            sel.all = attr.value.toJS();
        }
        else {
            sel.attributes.push(attr);
        }
        return selector;
    }),
    (($, pname, $0, v) => new Ast.InputParam($.location, pname.value, v)),
    (($, selector) => {
        const [sel, chan] = selector;
        return new Ast.Invocation($.location, sel, chan, [], null);
    }),
    (($, inv, ip) => {
        inv = inv.clone();
        inv.in_params.push(ip);
        return inv;
    }),
    (($, pname, $0, out_param) => new Ast.InputParam($.location, pname.value, out_param)),
    (($, pname, $0, $1) => new Ast.InputParam($.location, pname.value, new Ast.Value.Event(null))),
    (($, pname, $0, v) => new Ast.InputParam($.location, pname.value, v)),
    (($, pname) => new Ast.Value.VarRef(pname.value)),
    (($, pname) => [pname]),
    (($, list, $0, pname) => list.concat(pname)),
    (($, $0) => { return $0; }),
    (($, f1, $0, f2) => new Ast.BooleanExpression.And($.location, [f1, f2])),
    (($, $0) => { return $0; }),
    (($, f1, $0, f2) => new Ast.BooleanExpression.Or($.location, [f1, f2])),
    (($, $0) => { return $0; }),
    (($, $0, f) => new Ast.BooleanExpression.Not($.location, f)),
    (($, $0, pname) => new Ast.BooleanExpression.DontCare($.location, pname.value)),
    (($, pname, op, rhs) => {
        return new Ast.BooleanExpression.Compute($.location, new Ast.VarRefValue(pname.value), op, rhs);
    }),
    (($, lhs, op, rhs) => {
        return new Ast.BooleanExpression.Compute($.location, lhs, op, rhs);
    }),
    (($, fn, $0, filter, $1) => {
        const expr = new Ast.FilterExpression($.location, new Ast.InvocationExpression($.location, fn, null), filter, null);
        return new Ast.BooleanExpression.ExistentialSubquery($.location, expr);
    }),
    (($, value) => [value]),
    (($, list, $0, value) => {
        list.push(value);
        return list;
    }),
    (($, $0) => { return "=="; }),
    (($, $0) => { return ">="; }),
    (($, $0) => { return "<="; }),
    (($, $0) => { return "=~"; }),
    (($, $0) => { return "~="; }),
    (($, $0) => { return "starts_with"; }),
    (($, $0) => { return "ends_with"; }),
    (($, $0) => { return "prefix_of"; }),
    (($, $0) => { return "suffix_of"; }),
    (($, $0) => { return "contains"; }),
    (($, $0) => { return "contains~"; }),
    (($, $0) => { return "~contains"; }),
    (($, $0) => { return "in_array"; }),
    (($, $0) => { return "in_array~"; }),
    (($, $0) => { return "~in_array"; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, lhs, $0, rhs) => new Ast.Value.Computation('+', [lhs, rhs])),
    (($, lhs, $0, rhs) => new Ast.Value.Computation('-', [lhs, rhs])),
    (($, $0) => { return $0; }),
    (($, lhs, $0, rhs) => new Ast.Value.Computation('*', [lhs, rhs])),
    (($, lhs, $0, rhs) => new Ast.Value.Computation('/', [lhs, rhs])),
    (($, lhs, $0, rhs) => new Ast.Value.Computation('%', [lhs, rhs])),
    (($, $0) => { return $0; }),
    (($, lhs, $0, rhs) => new Ast.Value.Computation('**', [lhs, rhs])),
    (($, field, $0, value) => new Ast.Value.ArrayField(value, field.value)),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, value, $0, $1, filter, $2) => new Ast.Value.Filter(value, filter)),
    (($, $0, value, $1) => value),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, lhs, $0, rhs) => new Ast.Value.Computation('+', [lhs, rhs])),
    (($, lhs, $0, rhs) => new Ast.Value.Computation('-', [lhs, rhs])),
    (($, $0) => { return $0; }),
    (($, lhs, $0, rhs) => new Ast.Value.Computation('*', [lhs, rhs])),
    (($, lhs, $0, rhs) => new Ast.Value.Computation('/', [lhs, rhs])),
    (($, lhs, $0, rhs) => new Ast.Value.Computation('%', [lhs, rhs])),
    (($, $0) => { return $0; }),
    (($, lhs, $0, rhs) => new Ast.Value.Computation('**', [lhs, rhs])),
    (($, field, $0, value) => new Ast.Value.ArrayField(value, field.value)),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, value, $0, $1, filter, $2) => new Ast.Value.Filter(value, filter)),
    (($, op, $0, operand_list, $1) => new Ast.Value.Computation(op, operand_list)),
    (($, op, $0, value, $1) => new Ast.Value.Computation(op, [value])),
    (($, $0, $1, value, $2) => new Ast.Value.Computation('count', [value])),
    (($, $0) => new Ast.Value.Undefined(true)),
    (($, $0) => { return $0; }),
    (($, ctx) => new Ast.Value.ContextRef(ctx.value.name, ctx.value.type)),
    (($, $0, $1) => new Ast.Value.Array([])),
    (($, $0, values, $1) => new Ast.Value.Array(values)),
    (($, $0, $1) => new Ast.Value.Object({})),
    (($, $0, values, $1) => new Ast.Value.Object(values)),
    (($, name, $0, value) => ({ [name.value]: value })),
    (($, obj, $0, name, $1, value) => {
        obj[name.value] = value;
        return obj;
    }),
    (($, $0, $1, $2, rules, $3) => new Ast.Value.RecurrentTimeSpecification(rules)),
    (($, first) => [first]),
    (($, head, $0, tail) => head.concat([tail])),
    (($, $0, list, $1) => {
        const obj = {};
        for (const item of list) {
            if (obj[item[0]])
                return $.error(`Duplicate recurrent time rule key ${item[0]}`);
            obj[item[0]] = item[1];
        }
        return new Ast.RecurrentTimeRule(obj);
    }),
    (($, first) => [first]),
    (($, head, $0, tail) => head.concat([tail])),
    (($, $0, $1, v) => ['beginDate', v.value]),
    (($, $0, $1, v) => ['endDate', v.value]),
    (($, $0, $1, v) => ['beginTime', v]),
    (($, $0, $1, v) => ['endTime', v]),
    (($, $0, $1, v) => ['frequency', v]),
    (($, $0, $1, v) => ['interval', v]),
    (($, $0, $1, v) => ['dayOfWeek', v.value]),
    (($, $0, $1, $2) => ['subtract', true]),
    (($, $0, $1, $2) => ['subtract', false]),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, $0) => { return $0; }),
    (($, slot) => (slot.value === undefined ? new Ast.Value.Undefined(true) : slot.value)),
    (($, value) => [value]),
    (($, array, $0, v) => array.concat([v])),
    (($, $0) => new Ast.Value.Boolean(true)),
    (($, $0) => new Ast.Value.Boolean(false)),
    (($, $0) => new Ast.Value.String('')),
    (($, str) => new Ast.Value.String(str.value)),
    (($, $0, str, $1) => new Ast.Value.String(str)),
    (($, word) => word.value),
    (($, list, word) => list + ' ' + word.value),
    (($, num) => num.value),
    (($, num) => num.value),
    (($, $0) => 1),
    (($, $0) => 0),
    (($, num, unit) => new Ast.Value.Measure(num, unit.value)),
    (($, tok) => new Ast.Value.Measure(tok.value.value, tok.value.unit)),
    (($, tok) => new Ast.Value.Measure(tok.value.value, tok.value.unit)),
    (($, num) => new Ast.Value.Number(num)),
    (($, $0, num) => new Ast.Value.Number(-num)),
    (($, tok) => new Ast.Value.Currency(tok.value.value, tok.value.unit)),
    (($, $0, $1, $2, num, $3, unit, $4) => new Ast.Value.Currency(num, unit.value)),
    (($, num, tok) => new Ast.Value.Currency(num, tok.value)),
    (($, $0) => new Ast.Value.Location(new Ast.Location.Relative('current_location'))),
    (($, $0) => new Ast.Value.Location(new Ast.Location.Relative('home'))),
    (($, $0) => new Ast.Value.Location(new Ast.Location.Relative('work'))),
    (($, loc) => new Ast.Value.Location(new Ast.Location.Absolute(loc.value.latitude, loc.value.longitude, loc.value.display || null))),
    (($, $0, $1, str, $2) => new Ast.Value.Location(new Ast.Location.Unresolved(str))),
    (($, $0, $1, $2, lat, $3, lon, $4, display, $5) => {
        return new Ast.Value.Location(new Ast.Location.Absolute(lat.value, lon.value, display.value));
    }),
    (($, $0, $1, $2, lat, $3, lon, $4) => {
        return new Ast.Value.Location(new Ast.Location.Absolute(lat.value, lon.value, null));
    }),
    (($, $0, $1, $2, name, $3) => {
        return new Ast.Value.Location(new Ast.Location.Unresolved(name.value));
    }),
    (($, time) => new Ast.Time.Absolute(time.value.hour, time.value.minute, time.value.second || 0)),
    (($, time) => {
        const tokens = time.value.split(':');
        return new Ast.Time.Absolute(parseInt(tokens[0]), parseInt(tokens[1]), parseInt(tokens[2]));
    }),
    (($, time) => new Ast.Value.Time(time)),
    (($, $0) => new Ast.Value.Time(new Ast.Time.Relative('morning'))),
    (($, $0) => new Ast.Value.Time(new Ast.Time.Relative('evening'))),
    (($, $0) => new Ast.Value.Date(null)),
    (($, $0, $1, $2, $3) => new Ast.Value.Date(null)),
    (($, $0) => { return $0; }),
    (($, $0, $1, $2, year, $3, month, $4, day, $5) => {
        let yearnum = year.value;
        if (yearnum > 0 && yearnum < 100) {
            if (yearnum >= 50)
                yearnum = 1900 + yearnum;
            else
                yearnum = 2000 + yearnum;
        }
        const d = new Date;
        d.setFullYear(yearnum);
        // set both the month and the date at the same time
        // otherwise when today's date is the 31st and the chosen
        // month has 30 days, the Date will be adjusted to the
        // first day of the subsequent month, which is wrong
        d.setMonth(month.value - 1, day.value);
        d.setHours(0, 0, 0, 0);
        return new Ast.Value.Date(d);
    }),
    (($, $0, $1, $2, year, $3, month, $4, day, $5, hours, $6, minutes, $7, seconds, $8) => {
        const d = new Date;
        let yearnum = year.value;
        if (yearnum > 0 && yearnum < 100) {
            if (yearnum >= 50)
                yearnum = 1900 + yearnum;
            else
                yearnum = 2000 + yearnum;
        }
        d.setFullYear(yearnum);
        d.setMonth(month.value - 1, day.value);
        d.setHours(hours.value, minutes.value, seconds.value, 0);
        return new Ast.Value.Date(d);
    }),
    (($, $0, $1, $2, unix, $3) => {
        const d = new Date;
        d.setTime(unix.value);
        return new Ast.Value.Date(d);
    }),
    (($, $0, $1, $2, $3, iso, $4, $5) => {
        return new Ast.Value.Date(new Date(iso));
    }),
    (($, abs) => new Ast.Value.Date(date_utils_1.parseDate(abs.value))),
    (($, $0, $1, $2, year, $3, $4, $5, $6) => {
        return new Ast.Value.Date(new Ast.DatePiece(year.value, null, null, null));
    }),
    (($, $0, $1, $2, $3, month, $4, $5, $6) => {
        return new Ast.Value.Date(new Ast.DatePiece(null, month.value, null, null));
    }),
    (($, $0, $1, $2, $3, $4, day, $5, $6) => {
        return new Ast.Value.Date(new Ast.DatePiece(null, null, day.value, null));
    }),
    (($, $0, $1, $2, year, $3, month, $4, $5, $6) => {
        return new Ast.Value.Date(new Ast.DatePiece(year.value, month.value, null, null));
    }),
    (($, $0, $1, $2, $3, month, $4, day, $5, $6) => {
        return new Ast.Value.Date(new Ast.DatePiece(null, month.value, day.value, null));
    }),
    (($, $0, $1, $2, $3, $4, day, $5, time, $6) => {
        return new Ast.Value.Date(new Ast.DatePiece(null, null, day.value, time));
    }),
    (($, $0, $1, $2, $3, month, $4, day, $5, time, $6) => {
        return new Ast.Value.Date(new Ast.DatePiece(null, month.value, day.value, time));
    }),
    (($, $0, $1, $2, weekday, $3) => {
        const d = new Ast.WeekDayDate(weekday.value, null);
        return new Ast.Value.Date(d);
    }),
    (($, $0, $1, $2, weekday, $3, time, $4) => {
        const d = new Ast.WeekDayDate(weekday.value, time);
        return new Ast.Value.Date(d);
    }),
    (($, $0, unit) => new Ast.Value.Date(new Ast.DateEdge('start_of', unit.value))),
    (($, $0, unit) => new Ast.Value.Date(new Ast.DateEdge('end_of', unit.value))),
    (($, venum) => new Ast.Value.Enum(venum.value)),
    (($, entity) => new Ast.Value.Entity(entity.value.value, entity.value.type, entity.value.display)),
    (($, $0, str, $1, type) => {
        if (type.value === 'tt:hashtag' || type.value === 'tt:username')
            return new Ast.Value.Entity(str, type.value, null);
        else
            return new Ast.Value.Entity(null, type.value, str);
    }),
    (($, entity) => new Ast.Value.Entity(entity.value, 'tt:username', null)),
    (($, entity) => new Ast.Value.Entity(entity.value, 'tt:hashtag', null)),
    (($, entity) => new Ast.Value.Entity(entity.value, 'tt:url', null)),
    (($, entity) => new Ast.Value.Entity(entity.value, 'tt:phone_number', null)),
    (($, entity) => new Ast.Value.Entity(entity.value, 'tt:email_address', null)),
    (($, entity) => new Ast.Value.Entity(entity.value, 'tt:path_name', null)),
    (($, entity) => new Ast.Value.Entity(entity.value, 'tt:device', null)),
    (($, entity) => new Ast.Value.Entity(entity.value.kind + ':' + entity.value.channel, 'tt:function', null)),
    (($, entity) => new Ast.Value.Entity(entity.value, 'tt:picture', null)),
    (($, $0) => $0),
];
const $runtime = __importStar(require("./../utils/sr_parser_runtime"));
exports.default = $runtime.createParser({ TERMINAL_IDS, RULE_NON_TERMINALS, ARITY, GOTO, PARSER_ACTION, SEMANTIC_ACTION });
//# sourceMappingURL=parser.js.map